'''
Created on 7 sept. 2017

@author: Fab
'''

from lmtanalysis.Detection import *

import matplotlib
#matplotlib fix for mac (uncomment if needed)
#matplotlib.use('TkAgg' )


import matplotlib.pyplot as plt
from lmtanalysis.Chronometer import *
import matplotlib as mpl
import numpy as np
from mpl_toolkits.mplot3d import *
import matplotlib.ticker
import math
import time
from lmtanalysis.Measure import *
from statistics import *
from scipy.spatial import distance
from scipy.ndimage.measurements import standard_deviation
from statistics import mean
from lmtanalysis.Event import EventTimeLine
from lmtanalysis.Point import Point
from lmtanalysis.Mask import Mask
import matplotlib.patches as mpatches
from lxml import etree
import matplotlib.ticker as ticker
from lmtanalysis.Util import convert_to_d_h_m_s
from lmtanalysis.Util import getDatetimeFromFrame

idAnimalColor = [ None, "red","green","blue","orange"]

def getAnimalColor( animalId ):    
    return idAnimalColor[ animalId ]

class Animal():

    def __init__(self, baseId , RFID , name=None, genotype=None , user1 = None, age=None, sex=None, strain=None, cage=None, conn = None ):
        self.baseId = baseId
        self.RFID = RFID
        self.name = name
        self.genotype = genotype
        self.cage = cage
        self.user1 = user1
        self.age = age
        self.sex = sex
        self.strain = strain
        self.conn = conn
        self.detectionDictionnary = {}

    def setGenotype(self, genotype):
        self.genotype = genotype    
        cursor = self.conn.cursor()                    
        query = "UPDATE `ANIMAL` SET `GENOTYPE`='{}' WHERE `ID`='{}';".format( genotype, self.baseId )
        cursor.execute( query )            
        self.conn.commit()
        cursor.close()

    def setSex(self, sex):
        self.sex = sex    
        cursor = self.conn.cursor()   
        
#         #Check first if Sex exist in the Database, if not create it!
#         tables = []
# #         res = cursor.execute("SELECT name FROM sqlite_master WHERE type='table';")
#         res = cursor.execute("SELECT name FROM TABLE `ANIMAL`;")
 
#         for name in res:
#             print(name[0])
#             tables.append(name[0])

#         print("the 'tables' are:")
#         for t in tables:
#             print(t)
 
        # Check the number of row available in 'ANIMAL' base (Sqlite Table)
        query = "SELECT * FROM ANIMAL"
        cursor.execute( query )
        field_names = [i[0] for i in cursor.description]
        print ( "Fields available in lmtanalysis Z: " , field_names )
        
        if 'SEX' not in field_names:
            cursor.execute('ALTER TABLE `ANIMAL` ADD COLUMN SEX TEXT')
        
        query = "UPDATE `ANIMAL` SET `SEX`='{}' WHERE `ID`='{}';".format( sex, self.baseId )
        cursor.execute( query )            
        self.conn.commit()
        cursor.close()
        
    def setCage(self, cage):
        self.cage = cage    
        cursor = self.conn.cursor()   
        
#       Check first if Cage exist in the Database, if not create it!
 
        # Check the number of row available in 'ANIMAL' base (Sqlite Table)
        query = "SELECT * FROM ANIMAL"
        cursor.execute( query )
        field_names = [i[0] for i in cursor.description]
        # print ( "Fields available in lmtanalysis Z: " , field_names )
        
        if 'CAGE' not in field_names:
            cursor.execute('ALTER TABLE `ANIMAL` ADD COLUMN CAGE TEXT')
        
        query = "UPDATE `ANIMAL` SET `CAGE`='{}' WHERE `ID`='{}';".format( cage, self.baseId )
        cursor.execute( query )            
        self.conn.commit()
        cursor.close()
        
    def __str__(self):        
        return "Animal Id:{id} Name:{name} RFID:{rfid} Genotype:{genotype} User1:{user1}"\
            .format( id=self.baseId, rfid=self.RFID, name=self.name, genotype=self.genotype, user1=self.user1 )

    def getColor(self):
        return getAnimalColor( self.baseId )

    def getDetectionAt(self, t):
        if t in self.detectionDictionnary:
            return self.detectionDictionnary[t]
        return None

    def loadDetection(self, start=None, end=None, lightLoad = False ):
        '''
        lightLoad only loads massX and massY to speed up the load. Then one can only compute basic features such as global speed of the animals
        '''
        # print ( self.__str__(), ": Loading detection.")
        chrono = Chronometer("Load detection")

        self.detectionDictionnary.clear()
                
        cursor = self.conn.cursor()
        query =""
        if lightLoad == True :
            query = "SELECT FRAMENUMBER, MASS_X, MASS_Y FROM DETECTION WHERE ANIMALID={}".format( self.baseId )
        else:
            query = "SELECT FRAMENUMBER, MASS_X, MASS_Y, MASS_Z, FRONT_X, FRONT_Y, FRONT_Z, BACK_X, BACK_Y, BACK_Z,REARING,LOOK_UP,LOOK_DOWN FROM DETECTION WHERE ANIMALID={}".format( self.baseId )
            
        if ( start != None ):
            query += " AND FRAMENUMBER>={}".format(start )
        if ( end != None ):
            query += " AND FRAMENUMBER<={}".format(end )
            
        # print( query )
        cursor.execute( query )
        
        rows = cursor.fetchall()
        cursor.close()    
        
        for row in rows:
            frameNumber = row[0]
            massX = row[1]
            massY = row[2]
            detection = None
            #filter detection at 0

            if ( massX < 10 ):
                continue

            if not lightLoad:
                massZ = row[3]

                frontX = row[4]
                frontY = row[5]
                frontZ = row[6]
    
                backX = row[7]
                backY = row[8]
                backZ = row[9]
    
                rearing = row[10]
                lookUp = row[11]
                lookDown = row[12]
            
                detection = Detection( massX, massY, massZ, frontX, frontY, frontZ, backX, backY, backZ, rearing, lookUp, lookDown )
            else:
                detection = Detection( massX, massY , lightLoad = True )
            
            self.detectionDictionnary[frameNumber] = detection

        # print ( self.__str__(), " ", len( self.detectionDictionnary ) , " detections loaded in {} seconds.".format( chrono.getTimeInS( )) )
    
    def loadMask(self , frame ):
        self.setMask( self.getBinaryDetectionMask() )
        
    def clearMask(self):
        self.setMask( None )
    
    def getNumberOfDetection(self, tmin, tmax):
        
        
        return len ( self.detectionDictionnary.keys() )
    
    
    def filterDetectionByInstantSpeed(self , minSpeed, maxSpeed ):
        """
        speed function in LMT use t-1 and t+1 detection to provide a result.
        here we remove spurious tracking jump, so we check on t to t+1 frame.
        speed is in cm per second
        """
        nbRemoved = 0
        for key in sorted(self.detectionDictionnary.keys()):
            a = self.detectionDictionnary.get( key )
            b = self.detectionDictionnary.get( key+1 )
                        
            if ( b==None or a==None):
                continue
            
            speed = math.hypot( a.massX - b.massX, a.massY - b.massY )*scaleFactor/(1/30)
        
            if ( speed > maxSpeed or speed < minSpeed ):
                self.detectionDictionnary.pop( key )
                nbRemoved+=1
        
        # print( "Filtering Instant speed min:",minSpeed, "max:",maxSpeed, "number of detection removed:", nbRemoved )
    
    def filterDetectionByArea(self, x1, y1, x2, y2 ):
        '''
        filter detection in the cage ( using centimeter, starting at top left of the cage )
        '''
        nbRemoved = 0
        for key in sorted(self.detectionDictionnary.keys()):
            a = self.detectionDictionnary.get( key )

            if ( a==None):
                continue
            
            x = (a.massX - cornerCoordinates50x50Area[0][0] )* scaleFactor
            y = (a.massY - cornerCoordinates50x50Area[0][1] )* scaleFactor            
        
            if ( x < x1 or x > x2 or y < y1 or y > y2 ):
                self.detectionDictionnary.pop( key )
                nbRemoved+=1
        
        # print( "Filtering area, number of detection removed:", nbRemoved )
    
    def filterDetectionByEventTimeLine( self, eventTimeLineVoc ):
        '''
        filter detection using an event. Keep only what matches the event
        '''
        eventDic = eventTimeLineVoc.getDictionnary()
        nbRemoved = 0
        for key in sorted( self.detectionDictionnary.keys() ):
            a = self.detectionDictionnary.get( key )

            if ( a==None):
                continue

            if not ( key in eventDic ):
                self.detectionDictionnary.pop( key )        
                nbRemoved+=1
        
        # print( "Filtering area, number of detection removed:", nbRemoved )
        
    def clearDetection(self):
        
        self.detectionDictionnary.clear()
    
    def getMaxDetectionT(self):
        """
        returns the timepoint of the last detection.
        """
        
        if len ( self.detectionDictionnary.keys() ) == 0:
            return None
        
        return sorted(self.detectionDictionnary.keys())[-1]
    
    def getTrajectoryData( self , maskingEventTimeLine=None ):
        
        keyList = sorted(self.detectionDictionnary.keys())
        
        if maskingEventTimeLine!=None:
            keyList = maskingEventTimeLine.getDictionnary()
        
        xList = []
        yList = []
        
        previousKey = 0
        

        for key in keyList:
            
            #print ( "key:", key, "value", self.getSpeed( key ) , "previous:" , previousKey )
            
            if previousKey+1 != key:                
                xList.append( [np.nan, np.nan] )
                yList.append( [np.nan, np.nan] )
                previousKey = key

                #print("break previous")
                continue
            previousKey = key
            
            a = self.detectionDictionnary.get( key )
            if ( a==None):
                xList.append( [np.nan, np.nan] )
                yList.append( [np.nan, np.nan] )
                #print("break none A")
                continue
            b = self.detectionDictionnary.get( key+1 )
            if ( b==None):
                xList.append( [np.nan, np.nan] )
                yList.append( [np.nan, np.nan] )
                #print("break none B")
                continue
            
            xList.append( [a.massX,b.massX] )
            yList.append( [-a.massY,-b.massY] )
        
        return xList, yList
    
    
    def plotTrajectory(self , show=True, color='k', maskingEventTimeLine=None, title = "" ):
        
        print ("Draw trajectory of animal " + self.name )
        
        xList, yList = self.getTrajectoryData( maskingEventTimeLine )
            
        plt.plot( xList, yList, color=color, linestyle='-', linewidth=1, alpha=0.5, label= self.name )
        plt.title( title + self.RFID )
        plt.xlim(90, 420)
        plt.ylim(-370, -40)
        
        if ( show ):
            plt.show()    


    def plotTrajectory3D(self):
        
        print ("Draw 3D trajectory")
        keyList = sorted(self.detectionDictionnary.keys())
        
        mpl.rcParams['legend.fontsize'] = 10

        fig = plt.figure()
        ax = fig.gca(projection='3d')
        theta = np.linspace(-4 * np.pi, 4 * np.pi, 100)
        
        z = np.linspace(-2, 2, 100)
        r = z**2 + 1
        x = r * np.sin(theta)
        y = r * np.cos(theta)
        
        '''
        print ( z )
        print ( r )
        print ( x )
        print ( y )
        '''
        
        xList = []
        yList = []
        zList = []
    
        for key in keyList:
            
            a = self.detectionDictionnary.get( key )
            b = self.detectionDictionnary.get( key+1 )
            if ( b==None):
                continue
            
            xList.append( a.massX )
            yList.append( a.massY )
            zList.append( a.massZ )
            
        ax.plot(xList, yList, zList, label= "3D Trajectory of " + self.RFID )
        ax.legend()
        
        plt.show()
    
   
    def getDistancePerBin(self , binFrameSize , minFrame=0, maxFrame=None ):
        if ( maxFrame==None ):
            maxFrame= self.getMaxDetectionT()
        
        distanceList = []
        t = minFrame
        while ( t < maxFrame ):
            distanceBin = self.getDistance( t , t+binFrameSize )
            print( "Distance bin n:{} value:{}".format ( t , distanceBin ) )
            distanceList.append( distanceBin )
            t=t+binFrameSize
        
        return distanceList
     
        
    def getDistance(self , tmin=0, tmax= None ):
        '''
        Returns the distance traveled by the animal (in cm)        
        '''        
        print("Compute total distance min:{} max:{} ".format( tmin , tmax ))

        
        '''
        keyList = list( self.detectionDictionnary.keys() )
        if not alreadySorted:
            keyList = sorted(self.detectionDictionnary.keys())
        '''         
        if ( tmax==None ):
            tmax= self.getMaxDetectionT()
            
        totalDistance = 0
        for t in range( tmin , tmax ):
        #for key in keyList:
            
            '''
            if ( key <= tmin or key >= tmax ):
                continue
            '''

            a = self.detectionDictionnary.get( t )
            b = self.detectionDictionnary.get( t+1 )
                        
            if b==None or a==None:
                continue
            distance = math.hypot( a.massX - b.massX, a.massY - b.massY )
            if ( distance >85.5): #if the distance calculated between two frames is too large, discard
                continue
            
            totalDistance += distance
        
        totalDistance *= scaleFactor
            
        return totalDistance
    
    def getOrientationVector(self, t):
        
        d = self.detectionDictionnary.get( t )
        
        if d == None:
            return None

        if d.frontX == None:
            return None
        
        if d.backX == None:
            return None
        
        deltaX = d.frontX - d.backX
        deltaY = d.frontY - d.backY
        p = Point( deltaX, deltaY )
        return p

    def getSpeedVector(self, t):
        
        a = self.detectionDictionnary.get( t-1 )
        b = self.detectionDictionnary.get( t+1 )
        
        if a == None or b == None:
            return None
            
        speedVectorX = a.massX - b.massX
        speedVectorY = a.massY - b.massY

        p = Point( speedVectorX, speedVectorY )
        return p

    def getFrontSpeed(self, t):
        
        a = self.detectionDictionnary.get( t-1 )
        b = self.detectionDictionnary.get( t+1 )
        
        if a == None or b == None:
            return None
            
        speedVectorX = a.frontX - b.frontX
        speedVectorY = a.frontY - b.frontY

        p = Point( speedVectorX, speedVectorY )
        return p
    
    def getBackSpeed(self, t):
        
        a = self.detectionDictionnary.get( t-1 )
        b = self.detectionDictionnary.get( t+1 )
        
        if a == None or b == None:
            return None
            
        speedVectorX = a.backX - b.backX
        speedVectorY = a.backY - b.backY

        p = Point( speedVectorX, speedVectorY )
        return p
    
    
    def getDistanceSpecZone(self, tmin=0, tmax=None, xa=None, ya=None, xb=None, yb=None):

        keyList = sorted(self.detectionDictionnary.keys())
        
        if ( tmax==None ):
            tmax = self.getMaxDetectionT()
    
        distance = 0

        for key in keyList:
            
            if ( key <= tmin or key >= tmax ):
                #print ( 1 )
                continue
            
            a = self.detectionDictionnary.get( key )
            b = self.detectionDictionnary.get( key+1 )
            
            if ( b==None):
                continue
            
            if (a.massX<xa or a.massX>xb or a.massY<ya or a.massY>yb or b.massX<xa or b.massX>xb or b.massY<ya or b.massY>yb):
                #print ( 2 )
                continue
            
            if (math.hypot( a.massX - b.massX, a.massY - b.massY )>85.5): #if the distance calculated between two frames is too large, discard
                continue
            
            distance += math.hypot( a.massX - b.massX, a.massY - b.massY )
            
        distance *= scaleFactor
        
        return distance
    
    
    def getDistanceTo (self, t, animalB):
        '''
        determine the distance between the focal animal and another one specified in argument at one specified time point t
        check before that both animals are detected at this time point
        '''
        distanceTo = None

        if ( not ( t in animalB.detectionDictionnary ) ):
            return None

        if ( not ( t in self.detectionDictionnary ) ):
            return None
        
        if (animalB.detectionDictionnary[t].massX == None):
            return None
        
        if (math.hypot( self.detectionDictionnary[t].massX - animalB.detectionDictionnary[t].massX, self.detectionDictionnary[t].massY - animalB.detectionDictionnary[t].massY ) > 71*57/10): #if the distance calculated between the two individuals is too large, discard
            return None
        
        else:
            distanceTo = math.hypot( self.detectionDictionnary[t].massX - animalB.detectionDictionnary[t].massX, self.detectionDictionnary[t].massY - animalB.detectionDictionnary[t].massY )
            return distanceTo
        
        
    def getDistanceToPoint (self, t, xPoint, yPoint):
        '''
        determine the distance between the focal animal and a specific point in the arena at one specified time point t
        '''
        distanceToPoint = None

        if ( not ( t in self.detectionDictionnary ) ):
            return None
        
        if (math.hypot( self.detectionDictionnary[t].massX - xPoint, self.detectionDictionnary[t].massY - yPoint ) > MAX_DISTANCE_THRESHOLD): #if the distance calculated is too large, discard
            return None
        
        else:
            distanceToPoint = math.hypot( self.detectionDictionnary[t].massX - xPoint, self.detectionDictionnary[t].massY - yPoint )
            return distanceToPoint
            
    
    def getMeanBodyLength (self, tmin=0, tmax=None): 
        '''
        determine the mean body length over the time window specified
        '''
        keyList = sorted(self.detectionDictionnary.keys())
        
        if ( tmax==None ):
            tmax= self.getMaxDetectionT()
    
        if ( tmax==None ):
            self.meanBodyLength = None
            return self.meanBodyLength

        bodySizeList = []

        for key in keyList:
            
            if ( key <= tmin or key >= tmax ):
                continue
            
            a = self.detectionDictionnary.get( key )
            bodySizeList.append(a.getBodySize())
            
        mean = np.nanmean(bodySizeList)
        print( "mean animal bodysize: "  , mean )
        
        self.meanBodyLength = mean
        
        return self.meanBodyLength
    
    
    def getBodyThreshold (self, tmin=0, tmax=None): 
        '''
        determine the body size threshold used to determine SAP
        '''
        keyList = sorted(self.detectionDictionnary.keys())
        
        if ( tmax==None ):
            tmax= self.getMaxDetectionT()
    
        bodySizeList = []

        for key in keyList:
            
            if ( key <= tmin or key >= tmax ):
                continue
            
            a = self.detectionDictionnary.get( key )
            bodySizeList.append(a.getBodySize())
            
        #verifier si ce calcul tourne bien:
        threshold = np.nanmean(bodySizeList) + np.nanstd(bodySizeList)
        
        self.bodyThreshold = threshold
        
        return self.bodyThreshold
    
    
    def getMedianBodyHeight (self, tmin=0, tmax=None): 
        '''
        determine the body size threshold used to determine SAP
        '''
        keyList = sorted(self.detectionDictionnary.keys())
        
        if ( tmax==None ):
            tmax= self.getMaxDetectionT()
    
        bodyHeightList = []

        for key in keyList:
            
            if ( key <= tmin or key >= tmax ):
                continue
            
            a = self.detectionDictionnary.get( key )
            bodyHeightList.append(a.massZ)
            
        self.medianBodyHeight = np.median(np.array(bodyHeightList))
        
        return self.medianBodyHeight
    
    
    def getThresholdMassHeight (self, tmin=0, tmax=None): 
        '''
        determine the body size height threshold used to determine whether the animal is rearing or not
        here we use the 7th decile
        '''
        keyList = sorted(self.detectionDictionnary.keys())
        
        if ( tmax==None ):
            tmax= self.getMaxDetectionT()
    
        massHeightList = []

        for key in keyList:
            
            if ( key <= tmin or key >= tmax ):
                continue
            
            a = self.detectionDictionnary.get( key )
            massHeightList.append(a.massZ)
        
        decile = 7*len(massHeightList)/10
        print("7eme decile mass:")
        print(decile)
        print("arrondi massZ:")
        print(math.ceil(decile)-1)
            
        self.eightDecileMassHeight = sorted(massHeightList)[math.ceil(decile)-1]
        
        return self.eightDecileMassHeight
    
    
    def getThresholdFrontHeight (self, tmin=0, tmax=None): 
        '''
        determine the body size height threshold used to determine whether the animal is rearing or not
        here we use the 7th decile
        '''
        keyList = sorted(self.detectionDictionnary.keys())
        
        if ( tmax==None ):
            tmax= self.getMaxDetectionT()
    
        frontHeightList = []

        for key in keyList:
            
            if ( key <= tmin or key >= tmax ):
                continue
            
            a = self.detectionDictionnary.get( key )
            frontHeightList.append(a.frontZ)
        
        decile = 7*len(frontHeightList)/10
        print("7eme decile front:")
        print(decile)
        print("arrondi frontZ:")
        print(math.ceil(decile)-1)
            
        self.eightDecileFrontHeight = sorted(frontHeightList)[math.ceil(decile)-1]
        
        return self.eightDecileFrontHeight
    
    
    def getDirection(self, t):
        '''
        determines the direction of the animal using the head and the mass center
        '''
        a = self.detectionDictionnary.get( t )
        return a.getDirection();
            
    
    def getSpeed(self , t ):
        '''
        calculate the instantaneous speed of the animal at each frame
        '''
        a = self.detectionDictionnary.get( t-1 )
        b = self.detectionDictionnary.get( t+1 )
                        
        if ( b==None or a==None):
            return None
            
        speed = math.hypot( a.massX - b.massX, a.massY - b.massY )*scaleFactor/(2/30)
        
        return speed
    
    
    def getVerticalSpeed(self , t ):
        '''
        calculate the instantaneous vertical speed of the mass center of the animal at each frame
        '''
        a = self.detectionDictionnary.get( t-1 )
        b = self.detectionDictionnary.get( t+1 )
                        
        if ( b==None or a==None):
            return None
            
        verticalSpeed = (b.massZ - a.massZ)/2
        
        return verticalSpeed
    
    
    def getSap(self, tmin=0, tmax=None, xa=None, ya=None, xb=None, yb=None):
        print("Compute number of frames in SAP in specific zone of the cage")
    
        self.getBodyThreshold( )
        self.getMedianBodyHeight()
    
        #TODO: pas mettre body threshold en self car ... c'est pas la peine.
    
        keyList = sorted(self.detectionDictionnary.keys())
    
        if ( tmax==None ):
            tmax = self.getMaxDetectionT()
    
        sapList = []
        
        for key in keyList:
            
            if ( key <= tmin or key >= tmax ):
                #print ( 1 )
                continue
            
            detection = self.detectionDictionnary.get(key);
            if (detection.massX<xa or detection.massX>xb or detection.massY<ya or detection.massY>yb or detection.massZ>self.medianBodyHeight):
                #print ( 2 )
                continue
    
            speed = self.getSpeed( key )
            if ( speed == None ):
                #print ( 3 )
                continue
    
            if (detection.getBodySize( ) >=self.bodyThreshold and speed<SPEED_THRESHOLD_LOW and detection.massZ<self.medianBodyHeight):
                #print ( 4 )
                sapList.append( detection )

        return sapList
          
      
    def getSapDictionnary(self, tmin=0, tmax=None ):
        
        self.getBodyThreshold( )
        self.getMedianBodyHeight()
        
        keyList = sorted(self.detectionDictionnary.keys())
        
        if ( tmax==None ):
            tmax = self.getMaxDetectionT()
    
        sapDictionnary = {}
        
        for key in keyList:
            
            if ( key <= tmin or key >= tmax ):
                continue
            
            detection = self.detectionDictionnary.get(key);

            speed = self.getSpeed( key )
            if ( speed == None ):
                continue

            if (detection.getBodySize( ) >=self.bodyThreshold and speed<SPEED_THRESHOLD_LOW and detection.massZ<self.medianBodyHeight):
                sapDictionnary[key] = True

        return sapDictionnary
        
    
    def getCountFramesSpecZone(self, tmin=0, tmax=None, xa=None, ya=None, xb=None, yb=None):
        '''
        coordinates are in pixel
        '''
        keyList = sorted(self.detectionDictionnary.keys())
        
        x1=min( xa, xb )
        x2=max( xa, xb )
        y1=min( ya, yb )
        y2=max( ya, yb )
        
        if ( tmax==None ):
            tmax = self.getMaxDetectionT()
    
        count =0

        for key in keyList:
            
            if ( key <= tmin or key >= tmax ):
                #print ( 1 )
                continue
            
            detection = self.detectionDictionnary.get(key);
            if (detection.massX<x1 or detection.massX>x2 or detection.massY<y1 or detection.massY>y2):
                #print ( 2 )
                continue

            count+=1
            
        return count
    
    
    def plotDistance(self, color='k' , show=True ):
        print ("Plot distance")
        keyList = sorted(self.detectionDictionnary.keys())
        
        tList = []
        distanceList = []
    
        totalDistance = 0
        for key in keyList:
            
            a = self.detectionDictionnary.get( key )
            b = self.detectionDictionnary.get( key+1 )
                        
            if ( b==None):
                continue
            
            totalDistance += math.hypot( a.massX - b.massX, a.massY - b.massY )
            
            tList.append( key / 30 / 60 )
            distanceList.append( totalDistance )
        
        #fig,ax = plt.subplots()
        plt.plot( tList, distanceList, color=color, linestyle='-', linewidth=2 , label="Cumulated distance of " + self.__str__() )
        
        #formatter = matplotlib.ticker.FuncFormatter(lambda frame, x: time.strftime('%Hh%Mm%Ss', time.gmtime( frame // 30)))
        #formatter = matplotlib.ticker.FuncFormatter(lambda frame, x: time.strftime('%Hh%Mm', time.gmtime( frame // 30)))
        #ax.xaxis.set_major_formatter( formatter )                        
        #ax.xaxis.set_minor_locator( matplotlib.ticker.MultipleLocator(30*60) )
        #ax.xaxis.set_major_locator( matplotlib.ticker.MultipleLocator(30*60*10) )
        
        plt.legend()
        
        if ( show ):
            plt.show()        
    
    
    def getBinaryDetectionMask(self , t):
        '''
        returns the mask of a detection at a given T.
        '''
        query = "SELECT DATA FROM DETECTION WHERE ANIMALID={} AND FRAMENUMBER={}".format( self.baseId , t )

        cursor = self.conn.cursor()
        cursor.execute( query )
        
        rows = cursor.fetchall()
        cursor.close()    
        
        if ( len(rows)!=1 ):
            #print ("unexpected number of row: " , str( len(rows ) ) )
            return None
        
        row = rows[0]        
        data = row[0]
        
        mask = Mask( data, self.getColor( ) )
                
        return mask

    
class AnimalPool():
    """
    Manages an experiment.
    """
    
    def __init__(self):

        self.animalDictionnary = {}
        
    def getAnimalDictionnary(self):
        return self.animalDictionnary
    
    def getAnimalWithId(self , id):
        return self.animalDictionnary[id]
    
    def getAnimalList(self):
        animalList= []
        
        for k in self.animalDictionnary:
            animal = self.animalDictionnary[k] 
            animalList.append(animal)
        
        return animalList
        
    def loadAnimals( self, conn ):
        
        print ("Loading animals ...")
        
        cursor = conn.cursor()
        self.conn = conn        
    
        # Check the number of row available in 'ANIMAL' base (Sqlite Table)
        query = "SELECT * FROM ANIMAL"
        cursor.execute( query )
        field_names = [i[0] for i in cursor.description]
        print ( "Fields available in lmtanalysis Z: " , field_names )
        
        #build query
        query = "SELECT "
        nbField = len( field_names )
        if ( nbField == 3 ):
            query+="ID,RFID,NAME"
        elif ( nbField == 4 ):
            query+="ID,RFID,NAME,GENOTYPE"
        elif ( nbField == 5 ):
            print('There are 5 fields')
            for i in field_names:
                print(i)
            if "SEX" in field_names:
                print(" 'SEX' is in field_names")
                query+="ID,RFID,NAME,GENOTYPE,SEX"
            if "CAGE" in field_names:
                print(" 'CAGE' is in field_names")
                query+="ID,RFID,NAME,GENOTYPE,CAGE"
            else:
                query+="ID,RFID,NAME,GENOTYPE,IND"
        elif ( nbField == 6 ):
            print('There are 6 fields')
            for i in field_names:
                print(i)
            if "SEX" in field_names:
                print(" 'SEX' is in field_names")
                if "CAGE" in field_names:
                    print(" 'CAGE' is in field_names")
                    query+="ID,RFID,NAME,GENOTYPE,SEX,CAGE"
                else:
                    query+="ID,RFID,NAME,GENOTYPE,SEX,IND"
        elif ( nbField == 7 ):
            query+="ID,RFID,NAME,GENOTYPE,AGE,SEX,STRAIN"
        
        query += " FROM ANIMAL ORDER BY GENOTYPE"
        print ( "SQL Query: " + query )
        
        cursor.execute( query )
        rows = cursor.fetchall()
        cursor.close()    
        
        self.animalDictionnary.clear()        
                
        for row in rows:
            animal = None
            
            if len(row) == 3:
                animal = Animal(row[0], row[1], name=row[2], conn = conn )
            if len(row) == 4:
                animal = Animal(row[0], row[1], name=row[2], genotype=row[3] , conn = conn )
            if len(row) == 5:
                if row[4] == 'M' or row[4] == 'F': #Check if the sex is defined as 'M' or 'F' in row[4]
                    animal = Animal(row[0], row[1], name=row[2], genotype=row[3], sex=row[4], conn = conn )
                else: 
                    animal = Animal(row[0], row[1], name=row[2], genotype=row[3], user1=row[4], conn = conn )
            if len(row) == 6:
                animal = Animal(row[0], row[1], name=row[2], genotype=row[3], sex=row[4], cage = row[5], conn = conn )
            if len( row ) == 7:
                animal = Animal(row[0], row[1], name=row[2], genotype=row[3], age=row[4], 
                                sex=row[5], strain=row[6], conn = conn)
                
            if ( animal!= None):
                self.animalDictionnary[animal.baseId] = animal
                print ( animal )
            else:
                print ( "Animal loader : error while loading animal.")
    
    def getAnonymousDetection(self, frame ):
        if frame not in self.anonymousDetection:
            return None
        
        return self.anonymousDetection[frame] 
            
    def loadAnonymousDetection(self, start = None, end=None ):
        '''
        Load the dictionary of anonymous detection.
        each entry get a list of detection
        clear previous anonymous detection dictionary
        '''
        self.anonymousDetection = {}
        
        chrono = Chronometer("Load anonymous detection")                
        cursor = self.conn.cursor()
        
        #query = "SELECT FRAMENUMBER, MASS_X, MASS_Y FROM DETECTION WHERE ANIMALID IS NULL"
        query = "SELECT FRAMENUMBER, MASS_X, MASS_Y FROM DETECTION WHERE ANIMALID IS NULL"
        
        if ( start != None ):
            query += " AND FRAMENUMBER>={}".format(start )
        if ( end != None ):
            query += " AND FRAMENUMBER<={}".format(end )
            
        print( query )
        cursor.execute( query )
        
        rows = cursor.fetchall()
        cursor.close()    
        
        for row in rows:
            frameNumber = row[0]
            massX = row[1]
            massY = row[2]
            #data = row[3]
            detection = None
            #filter detection at 0
            if ( massX < 10 ):
                continue

            detection = Detection( massX, massY , lightLoad = True )
            #detection.setMask( Mask( data ) )
            
            if frameNumber not in self.anonymousDetection:
                self.anonymousDetection[frameNumber] = []
                
            self.anonymousDetection[frameNumber].append( detection )
                            
        print ( len( self.anonymousDetection ) , " frames containing anonymous detections loaded in {} seconds.".format( chrono.getTimeInS( )) )
    
      
    def loadDetection (self , start = None, end=None , lightLoad = False ):
        for animal in self.animalDictionnary.keys():
            self.animalDictionnary[animal].loadDetection( start = start, end = end , lightLoad=lightLoad )

    def filterDetectionByInstantSpeed(self, minSpeed, maxSpeed):
        for animal in self.animalDictionnary.keys():
            self.animalDictionnary[animal].filterDetectionByInstantSpeed( minSpeed, maxSpeed )

    def filterDetectionByArea(self, x1, y1, x2, y2 ):
        for animal in self.animalDictionnary.keys():
            self.animalDictionnary[animal].filterDetectionByArea( x1, y1, x2, y2 )

    def filterDetectionByEventTimeLine(self, eventTimeLineVoc ):
        for animal in self.animalDictionnary.keys():
            self.animalDictionnary[animal].filterDetectionByEventTimeLine( eventTimeLineVoc )

    def getGenotypeList(self):
        
        genotype = {}
        
        for k in self.animalDictionnary:
            animal = self.animalDictionnary[k] 
            genotype[animal.genotype] = True
                
        return genotype.keys()
    
    def getAnimalsWithGenotype(self, genotype ):
        
        resultList= []
        
        for k in self.animalDictionnary:
            animal = self.animalDictionnary[k] 
            if ( animal.genotype == genotype ):
                resultList.append( animal )
                
        return resultList
        
        #return [x for x in self.animalDictionnary if x.genotype==genotype ]

    def getNbAnimals(self):
        return len(self.animalDictionnary)
    
    def getMaxDetectionT(self):
        """
        returns the timepoint of the last detection of all animals
        """
        maxFrame = 0
        for animal in self.getAnimalList():
            maxFrame = max( maxFrame, animal.getMaxDetectionT() )
        
        return maxFrame
    
    def frameToTimeTicker(self, x, pos):
        vals= convert_to_d_h_m_s( x )
        experimentTime = "D{0} - {1:02d}:{2:02d}".format( int(vals[0])+1, int(vals[1]), int(vals[2]) )
       
        realTime = ""
        if x > 0:
            datetime = getDatetimeFromFrame( self.conn , x )
            if datetime != None:
                realTime = "\n" + getDatetimeFromFrame( self.conn , x ).strftime('%d-%b-%Y %H:%M:%S')
        
        return experimentTime + realTime

    def plotSensorData( self , sensor="TEMPERATURE", show = True , saveFile = None , minValue=0 , autoNight= False):
        """
        plots data for temperature
        """
        print( "plotting sensor data. " , sensor )
        cursor = self.conn.cursor()
    
        # Check the number of row available in base
        query = "SELECT FRAMENUMBER,"+sensor+" FROM FRAME"
        try:
            cursor.execute( query )
        except:
            print("plot sensor data: can't access data for ", sensor )
            return
            
        rows = cursor.fetchall()
        cursor.close()    
        
        frameNumberList = []
        sensorValueList = []        
        for row in rows:
            
            sensorValue = row[1]
            if ( sensorValue > minValue ):
                if sensorValue != None and minValue !=None:
                    frameNumberList.append( row[0] )
                    sensorValueList.append( sensorValue )

        fig, ax = plt.subplots( nrows = 1 , ncols = 1 , figsize=( 24, 8 ) )
        
        ''' set x axis '''
        formatter = ticker.FuncFormatter( self.frameToTimeTicker )
        ax.xaxis.set_major_formatter(formatter)
        
        ax.tick_params(labelsize=20 )
        ax.xaxis.set_major_locator(ticker.MultipleLocator( 30 * 60 * 60 * 12 ))
        ax.xaxis.set_minor_locator(ticker.MultipleLocator( 30 * 60 * 60 ))
        
        ax.plot( frameNumberList, sensorValueList, color= "black" , linestyle='-', linewidth=1, label= sensor )
        ax.legend( loc=1 )
        
        # plot night
        nightTimeLine = EventTimeLine( self.conn, "night" , None, None, None , None )        
        
        mean = 0
        std = 0
        
        if len( sensorValueList ) > 1 :
            mean = np.mean( sensorValueList )
            std = np.std( sensorValueList )
        
        for nightEvent in nightTimeLine.eventList:
            
            ax.axvspan( nightEvent.startFrame, nightEvent.endFrame, alpha=0.1, color='black', ymin= 0, ymax = 1 )
            ax.text( (nightEvent.startFrame+ nightEvent.endFrame)/2 , mean+std , "dark phase" ,fontsize=20,ha='center')

        autoNightList = []
        if autoNight:
            # compute mean value:
            plt.axhline(y=mean, color='r', linestyle='--' )
            ax.text( 0 , mean , "auto night threshold" ,fontsize=20, ha='left' , va='bottom')
            
            inNight=False
            start = None
            end = None
            for i in range(len(frameNumberList)):
                frame = frameNumberList[i]
                value = sensorValueList[i]
                if value < mean: # night
                    if inNight:
                        continue
                    else:
                        inNight = True
                        start=frame
                else:
                    if inNight:
                        end = frame
                        inNight = False
                        if end-start > 300:
                            autoNightList.append( ( start, end ) )
                        else:    
                            print("Skipping very short night phase. (less than 10 seconds)")
                        start = None
                        end = None
            
            for autoNight in autoNightList:
                
                ax.axvspan( autoNight[0], autoNight[1], alpha=0.1, color='red', ymin= 0.2, ymax = 0.8 )
                ax.text( (autoNight[0]+autoNight[1])/2 , mean , "auto dark phase" ,fontsize=20,ha='center', color='red')
        
        plt.xlabel('time')
        
        if sensor == "TEMPERATURE":
            plt.ylabel('Temperature (C)')
            plt.ylim( 17, 28 )
            
        
        if sensor == "SOUND":
            plt.ylabel('Sound level (indice)')

        if sensor == "HUMIDITY":
            plt.ylabel('Humidity %')
            plt.ylim( 0, 100 )

        if sensor == "LIGHTVISIBLE":
            plt.ylabel('Visible light (indice)')

        if sensor == "LIGHTVISIBLEANDIR":
            plt.ylabel('Visible and infrared light (indice)')

        if saveFile !=None:
            print("Saving figure : " + saveFile )
            fig.savefig( saveFile, dpi=100)
        
        '''
        print("TEST 1")
        import os
        os.startfile( saveFile, 'open')
        print( "TEST 2")
        '''
            
        if ( show ):
            plt.show()
        plt.close()
        
        if autoNight:
            return autoNightList

        
    def plotNight(self, show = True , saveFile = None ):
        
        fig, ax = plt.subplots( nrows = 1 , ncols = 1 , figsize=( 12, 4 ) )

        nightTimeLine = EventTimeLine( self.conn, "night" , None, None, None , None )        
        for nightEvent in nightTimeLine.eventList:
            
            ax.axvspan( nightEvent.startFrame, nightEvent.endFrame, alpha=0.1, color='black')
            ax.text( (nightEvent.startFrame+ nightEvent.endFrame)/2 , 0.5 , "dark phase" ,fontsize=8,ha='center')
        
        ''' set x axis '''
        formatter = ticker.FuncFormatter( self.frameToTimeTicker )
        ax.xaxis.set_major_formatter(formatter)
        ax.tick_params(labelsize=6 )
        ax.xaxis.set_major_locator(ticker.MultipleLocator( 30 * 60 * 60 * 12 ))
        ax.xaxis.set_minor_locator(ticker.MultipleLocator( 30 * 60 * 60 ))

        if saveFile !=None:
            print("Saving figure : " + saveFile )
            fig.savefig( saveFile, dpi=100)
            
        if ( show ):
            plt.show()
        plt.close()
        
    def buildSensorData(self, file , show=False ):
        print("Build sensor data")
        #self.plotNight( show = show, saveFile = file+"_day night.pdf" )
        self.plotSensorData( sensor = "TEMPERATURE" , minValue = 10, saveFile = file+"_log_temperature.pdf", show = show )
        self.plotSensorData( sensor = "SOUND" , saveFile = file+"_log_sound level.pdf" , show = show )
        self.plotSensorData( sensor = "HUMIDITY" , minValue = 5 , saveFile = file+"_log_humidity.pdf" ,show = show )        
        self.plotSensorData( sensor = "LIGHTVISIBLE" , minValue = 40 , saveFile = file+"_log_light visible.pdf", show = show  )
        self.plotSensorData( sensor = "LIGHTVISIBLEANDIR" , minValue = 50 , saveFile = file+"_log_light visible and infra.pdf", show = show  )

 
    def plotTrajectory( self , show=True, maskingEventTimeLine=None , title = None, scatter = False, saveFile = None ):
        
        print( "AnimalPool: plot trajectory.")
        nbCols = len( self.getAnimalList() )+1
        fig, axes = plt.subplots( nrows = 1 , ncols = nbCols , figsize=( nbCols*4, 1*4 ) , sharex='all', sharey='all'  )
        
        if title==None:
            title="Trajectory of animals"
        
        #draw all animals
        axis = axes[0]
        legendList=[]
        for animal in self.getAnimalList():
            
            print ("Compute trajectory of animal " + animal.name )
            xList, yList = animal.getTrajectoryData( maskingEventTimeLine )                        
            print ("Draw trajectory of animal " + animal.name )
            if scatter == True:
                axis.scatter( xList, yList, color= animal.getColor() , s=1 , linewidth=1, alpha=0.05, label= animal.RFID )
                legendList.append( mpatches.Patch(color=animal.getColor(), label=animal.RFID) )
            else:
                axis.plot( xList, yList, color= animal.getColor() , linestyle='-', linewidth=1, alpha=0.5, label= animal.RFID )
                
        axis.legend( handles = legendList , loc=1 )
        axis.set_xlim(90, 420)
        axis.set_ylim(-370, -40)
        
        #draw separated animals
        for animal in self.getAnimalList():
            axis = axes[self.getAnimalList().index(animal)+1]
            
            legendList=[]
            
            print ("Compute trajectory of animal " + animal.name )
            xList, yList = animal.getTrajectoryData( maskingEventTimeLine )                        
            print ("Draw trajectory of animal " + animal.name )
            if scatter == True:
                axis.scatter( xList, yList, color= animal.getColor() , s=1 , linewidth=1, alpha=0.05, label= animal.RFID )
                legendList.append( mpatches.Patch(color=animal.getColor(), label=animal.RFID) )
            else:
                axis.plot( xList, yList, color= animal.getColor() , linestyle='-', linewidth=1, alpha=0.5, label= animal.RFID )
                
            axis.legend( handles = legendList , loc=1 )
        
        fig.suptitle( title )
        
        if saveFile !=None:
            print("Saving figure : " + saveFile )
            fig.savefig( saveFile, dpi=100)
            
        if ( show ):
            plt.show()
        plt.close()
    
    def showMask(self, t ):
        '''
        show the mask of all animals in a figure
        '''
        
        fig, ax = plt.subplots()
        ax.set_xlim(90, 420)
        ax.set_ylim(-370, -40)

        for animal in self.getAnimalList():                    
            mask = animal.getBinaryDetectionMask( t )
            mask.showMask( ax=ax )
        
        plt.show()
        
    def getParticleDictionnary(self , start, end ):
        '''
        return the number of particle per frame
        '''  
        
        query = "SELECT * FROM FRAME WHERE FRAMENUMBER>={} AND FRAMENUMBER<={}".format( start, end )
        
        print ( "SQL Query: " + query )

        cursor = self.conn.cursor()        
        cursor.execute( query )
        rows = cursor.fetchall()
        cursor.close()
        
        particleDictionnary = {}    
                        
        for row in rows:
            
            particleDictionnary[ row[0] ] = row[2]
            
        return particleDictionnary
        
    





'''
Created on 6 sept. 2017

@author: Fab
'''
import sqlite3
from time import *
from lmtanalysis.Chronometer import Chronometer
from lmtanalysis.Animal import *
from lmtanalysis.Detection import *
from lmtanalysis.Measure import *
import matplotlib.pyplot as plt
import numpy as np
from lmtanalysis.Event import *
from lmtanalysis.Measure import *
from lmtanalysis.EventTimeLineCache import EventTimeLineCached
from lmtanalysis.BehaviouralSequencesUtil import exclusiveEventList

def flush( connection ):
    ''' flush event in database '''
    for exclusiveEvent in exclusiveEventList[-3:-1]:
        deleteEventTimeLineInBase(connection, exclusiveEvent)

def reBuildEvent( connection, file, tmin=None, tmax=None , pool = None ):
    moveEventList = ['Move isolated', 'Stop isolated']

    moveEventListExclusive = exclusiveEventList[-3:-1]

    eventPairs = {'Move isolated': 'Move isolated exclusive',
                  'Stop isolated': 'Stop isolated exclusive'}

    if ( pool == None ):
        pool = AnimalPool( )
        pool.loadAnimals( connection )

    ''' load the existing timelines for animal '''
    timeLine = {}
    dicoContact = {}
    for event in moveEventList+exclusiveEventList[:-3]:
        timeLine[event] = {}
        dicoContact[event] = {}
        for animal in range(1, pool.getNbAnimals() + 1):
            timeLine[event][animal] = EventTimeLineCached( connection, file, event, animal, minFrame=tmin, maxFrame=tmax )
            dicoContact[event][animal] = timeLine[event][animal].getDictionary(minFrame=tmin, maxFrame=tmax)

    ''' create the timelines of the different exclusive moves '''
    timeLineExclusive = {}
    for event in moveEventListExclusive:
        timeLineExclusive[event] = {}
        for animal in range(1, pool.getNbAnimals() + 1):
            timeLineExclusive[event][animal] = EventTimeLine(None, event, animal, loadEvent=False)

    #generate the dico of the different move events and initiate the list of frames to remove from each timeline
    moveDicoExclusive = {}
    framesToRemove = {}
    for event in moveEventList:
        moveDicoExclusive[eventPairs[event]] = {}
        framesToRemove[eventPairs[event]] = {}
        for animal in range(1, pool.getNbAnimals() + 1):
            moveDicoExclusive[eventPairs[event]][animal] = timeLine[event][animal].getDictionary(minFrame=tmin, maxFrame=tmax)
            framesToRemove[eventPairs[event]][animal] = []

    ###########################################################################
    #exclude the move and stop events where animals are in contacts
    for animal in range(1, pool.getNbAnimals() + 1):
        for moveEvent in exclusiveEventList[-3:-1]:
            for t in moveDicoExclusive[moveEvent][animal].keys():
                for contactEvent in exclusiveEventList[:-3]:
                    if t in dicoContact[contactEvent][animal].keys():
                        print('t = ', t, 'in', moveEvent, ' and in ', contactEvent)
                        framesToRemove[moveEvent][animal].append(t)

    ###########################################################################
    # clean the dictionary of the move and stop events from frames that are overlapping with exclusive contacts
    for animal in range(1, pool.getNbAnimals() + 1):
        for moveEvent in exclusiveEventList[-3:-1]:
            for t in framesToRemove[moveEvent][animal]:
                moveDicoExclusive[moveEvent][animal].pop(t, None)

    #####################################################
    #reduild all events based on dictionary
    for moveEvent in exclusiveEventList[-3:-1]:
        for animal in range(1, pool.getNbAnimals() + 1):
            timeLineExclusive[moveEvent][animal].reBuildWithDictionnary(moveDicoExclusive[moveEvent][animal])
            timeLineExclusive[moveEvent][animal].endRebuildEventTimeLine(connection)

    # log process
    from lmtanalysis.TaskLogger import TaskLogger
    t = TaskLogger( connection )
    t.addLog( "Build Event Exclusive move and stop events" , tmin=tmin, tmax=tmax )
                       
    # print( "Rebuild event finished." )
        
    


import sys
sys.path.insert(1, "../")

from IPython.display import Markdown, display, clear_output
import ipywidgets as widgets
from ipywidgets import *
from tkinter.filedialog import askopenfilename
from lmtanalysis.FileUtil import getCsvFileToProcess
import matplotlib
import matplotlib.pyplot as plt
# from matplotlib.patches import Patch
import numpy as np
import pandas as pd
import seaborn as sns
import dabest
import random
import re
import os
import csv
import itertools
import scipy
from scipy import stats
from scipy.stats import wilcoxon
import statsmodels
import statsmodels.api as sm
import statsmodels.formula.api as smf
from statsmodels.formula.api import ols
from tabulate import tabulate
import warnings

warnings.filterwarnings('ignore')

# global df
# global genos
# global selected_genos
# global pivot_df2
# global merged_df
# global selected_genos
# global temp_df_loop1
# global df_new

pd.set_option('display.max_columns', None)
pd.set_option('max_colwidth', None)

# Read csv
path = getCsvFileToProcess()
df = pd.read_csv(path)

# Replacement of injections
remplacement = {'weekend1': '(1)weekend1', '1-3NaCl': '(2)3NaCl',
                '2-1Amphet': '(3)1Amphet', '3-1Amphet': '(4)1Amphet',
                '4-1Amphet': '(5)1Amphet', 'weekend2': '(6)weekend2',
                '5-3Amphet': '(7)3Amphet', '6-3Amphet': '(8)3Amphet',
                '7-3Amphet': '(9)3Amphet', 'LMT1-2mo - Copy': 'LMT1-2mo',
                'LMT2-3mo - Copy' : 'LMT2-3mo', 'LMT3-4mo - Copy' : 'LMT3-4mo',
                'LMT1-2 mo-(3 nights) - Copy' : 'LMT1-2mo', 'LMT1-2mo-(EXTRA DAY) - Copy' : 'LMT1-2mo'}

# Using the "replace" method to replace values
df['Injection'] = df['Injection'].replace(remplacement)

# Ipywidgets buttons arrangement
style = {'description_width': '100px'}
layout = widgets.Layout(width='300px',
                        height='25px')
# Buttons 'reset selection'
plot_reset = widgets.Button(description='Reset selection')
stat_reset = widgets.Button(description='Reset selection')

# Tab1 : Selection of behaviors to analyze
animalnumber = widgets.SelectMultiple(options=['Isolated beh. (eg:Rearing, SAP,...)', 'Beh. involving 2 mice',
                                               'Beh. involving 3 mice', 'Beh. involving 4 mice'],
                                      rows=4,
                                      description='Behaviors to analyze',
                                      style={'description_width': '120px'},
                                      layout=widgets.Layout(width='330px'),
                                      disabled=False)

# Selection of analysis type "Number" or "Duration"
data_type = ["Number of events", "Event duration"]
choicetype = widgets.Dropdown(options=data_type,
                              value=data_type[0],
                              description='Type',
                              style={'description_width': '120px'},
                              disable=False)

# Definition of variables
date = df["Date"].unique()


# Cage defs
# Custom sort function
def cle_tri_inj(element):
    # Use a regular expression to find the first digit in the element
    match = re.search(r'\d', element)

    if match:
        # If a digit is found, assign it to the first digit
        first_digit = int(match.group())

        # Use a second regular expression to find the second digit
        match2 = re.search(r'(\d+)(?=\D*$)', element)
        second_digit = int(match2.group()) if match2 else 0

        # Return a tuple to sort based on the two digits
        return (first_digit, second_digit)
    else:
        # If no digit is found, assign 0 for both digits
        return (0, 0)


injection = sorted(df["Injection"].unique(), key=cle_tri_inj)


# injection = sorted(df["Injection"].unique())

# Cage defs
# Custom sort function
# def cle_tri_cage(element):
#     # Separate the prefix "Cage" from the number
#     prefixe, nombre = element.split("Cage")
#     return (int(nombre), prefixe)

def cle_tri_cage(element):
    # Extract the number after "Cage"
    nombre_str = element[len("Cage"):]

    # Convert the integer number for to sort
    return int(nombre_str)


# Sort the list using the custom sort function
cage = sorted(df["Cage"].unique(), key=cle_tri_cage)
# cage = df["Cage"].unique()
night_phase = sorted(df["Night-Phase"].unique())
event = df["name"].unique()
stats = ['Linear Mixed Model', 'ANOVA']

# Create widgets
# drop_injection_plot = selection of injection day for plots
drop_injection_plot = widgets.SelectMultiple(options=injection,
                                             rows=len(injection),
                                             description='Injection',
                                             disabled=False)

# drop_cage_plot = selection of cage for plots
drop_cage_plot = widgets.SelectMultiple(options=cage,
                                        rows=5,
                                        description='Cage',
                                        disabled=False)

# drop_event = selection of behavior for plots
drop_event = widgets.Dropdown(options=event,
                              value=event[1],
                              description='Event:',
                              disabled=False)

# # drop_night_plot = selection of night for plots
# drop_night_plot = widgets.SelectMultiple(options=night_phase,
#                                          rows=5,
#                                          description='Night_Phase',
#                                          disabled=False)

# drop_stat = selection of stats type (currently only the Mixed-Model)
drop_stat = widgets.Dropdown(options=stats,
                             value=stats[0],
                             description='Statistic:',
                             style=style,
                             disabled=False)

# drop_injection_plot = selection of injection day for stats
drop_injection_stat = widgets.SelectMultiple(options=injection,
                                             rows=len(injection),
                                             description='Injection',
                                             style=style,
                                             disabled=False)

# drop_cage_plot = selection of cage for stats
drop_cage_stat = widgets.SelectMultiple(options=cage,
                                        rows=5,
                                        description='Cage',
                                        style=style,
                                        disabled=False)

# # drop_night_stat = selection of night for stats
# drop_night_stat = widgets.SelectMultiple(options=night_phase,
#                                          rows=5,
#                                          description='Night_Phase',
#                                          style=style,
#                                          disabled=False)

# Create widget for folder name
nom_dossier_plots = widgets.Text(placeholder='Type here',
                           description='Folder name:',
                           style=style,
                           disabled=False)

# Create widget for folder name
nom_dossier_stats = widgets.Text(placeholder='Type here',
                           description='Folder name:',
                           style = style,
                           disabled=False)

# Create button widget for plots
plotbutton = widgets.Button(description="Plots exportation",
                            layout = layout,
                            style = style)

# Create button widget for stats plots
statbutton = widgets.Button(description="Stats plots exportation",
                            layout=layout,
                            style=style)

# Create IntRangeSlider widget
range_slide_plot = widgets.IntRangeSlider(value=[0, max(df['Bin'])],
                                     min=0,
                                     max=max(df['Bin']),step=1,
                                     description='Bins :',
                                     disabled=False,
                                     continuous_update=False,
                                     orientation='horizontal',
                                     readout=True,
                                     readout_format='d')

# Create IntRangeSlider widget
range_slide_stats = widgets.IntRangeSlider(value=[0, max(df['Bin'])],
                                     min=0,
                                     max=max(df['Bin']),step=1,
                                     description='Bins :',
                                     disabled=False,
                                     continuous_update=False,
                                     orientation='horizontal',
                                     readout=True,
                                     readout_format='d')

# Create button widget for data retrieval
recup_data = widgets.Button(description="Data exportation",
                            layout = layout,
                            style = style)

# Genotype selection
genos = widgets.SelectMultiple(options=df.GenoA.unique(),
                               rows=len(df.GenoA.unique()),
                               description='Genotypes',
                               style = style,
                               disabled=False)

selected_genos = genos.value

def get_data_stats(recup_data):
    current_directory = os.getcwd()
    # Create the full path to the 'results' folder
    results_directory = os.path.join(current_directory, 'results')

    # Ensure that the 'results' folder exists, if not, create it
    if not os.path.exists(results_directory):
        os.makedirs(results_directory)
    # Create the full path to the 'result_df.csv' file in the 'results' folder
    result_df.to_csv(os.path.join(results_directory, 'result_df.csv'), index=False)

recup_data.on_click(get_data_stats)

# Assign the function 'results_update_dropdown' to the 'plotbutton' button
def get_plots_button(plotbutton):
    interactive_results_plots = widgets.interactive_output(results_update_dropdown,
                                                           {'date': drop_injection_plot,
                                                            'cage': drop_cage_plot,
                                                            # 'night_phase': drop_night_plot,
                                                            'event': drop_event,
                                                            'range_slide_plot': range_slide_plot})

# Associate the on_button_click function with the button click event
plotbutton.on_click(get_plots_button)

# Assign the function 'results_update_stats' to the 'statbutton' button
def get_plots_stats_button(statbutton):
    interactive_results_stats = widgets.interactive_output(results_update_stats,
                                                           {'date': drop_injection_stat,
                                                            'genos': genos,
                                                            'cage': drop_cage_stat,
                                                            # 'night_phase': drop_night_stat,
                                                            'event': drop_event,
                                                            'choice_type': choicetype,
                                                            'range_slide_stats': range_slide_stats})


# Associate the on_button_click function with the button click event
statbutton.on_click(get_plots_stats_button)

output = widgets.Output()

# Associate widgets with tabs
box1 = VBox([plot_reset,
             range_slide_plot,
             drop_injection_plot,
             drop_cage_plot,
             # drop_night_plot,
             drop_event,
             nom_dossier_plots,
             plotbutton])
box2 = VBox([stat_reset,
             range_slide_stats,
             genos,
             # drop_stat,
             drop_injection_stat,
             drop_cage_stat,
             # drop_night_stat,
             drop_event,
             nom_dossier_stats,
             statbutton,
             recup_data])

# Create the tab for the widgets
tab0 = widgets.VBox(children=[animalnumber, choicetype])
tab1 = widgets.Tab(children=[box1])
tab2 = widgets.VBox(children=[box2])
tab1.children = (tab0,) + tab1.children + tab2.children
tab1.set_title(0, 'Animals')
tab1.set_title(1, 'Plots')
tab1.set_title(2, 'Stats')

# Display the tab
# display(tab1)

# List of existing events in the current databases
event_options = {
    'Isolated beh. (eg:Rearing, SAP,...)': ["Move isolated", "Rearing", "Rear isolated", "Stop isolated", "SAP", "Distance"],
    'Beh. involving 2 mice': ["Contact", "Oral-oral Contact", "Oral-genital Contact", "Side by side Contact",
                              "Side by side Contact, opposite way", "Social approach", "Social escape",
                              "Approach contact", "Approach rear", "Break contact", "Get away",
                              "FollowZone Isolated", "Train2", "Group2"],
    'Beh. involving 3 mice': ["Group3"],
    'Beh. involving 4 mice': ["Group4", "Nest3", "Nest4"]}


# Update the events (drop_event) based on the selection of the 'animalnumber' widget
def update_event_options(change):
    selected_animalnumber = animalnumber.value
    selected_options = []
    for behavior in selected_animalnumber:
        selected_options += event_options[behavior]
    drop_event.options = selected_options


animalnumber.observe(update_event_options, names='value')


# Function to update selections in the 'Plot' tab after clicking the 'plot_reset' button
def reset_plot_selections(plot_reset):
    drop_injection_plot.value = ()
    drop_cage_plot.options = df['Cage'].unique()
    # drop_night_plot.options = df['Night-Phase'].unique()


plot_reset.on_click(reset_plot_selections)


# Function to update selections in the 'Stat' tab after clicking the 'stat_reset' button
def reset_stats_selections(stat_reset):
    drop_injection_stat.value = ()
    drop_cage_stat.options = df['Cage'].unique()
    # drop_night_stat.options = df['Night-Phase'].unique()


stat_reset.on_click(reset_stats_selections)

# To select multiple dates
def update_options_plot(change):
    # Filter DataFrame based on selected values in drop_injection_plot
    filtered_df = df[df['Injection'].isin(drop_injection_plot.value)]

    # Update drop_cage_plot options with unique values from the "Cage" column for filtered data
    cage_options_plot = filtered_df['Cage'].unique()
    drop_cage_plot.options = cage_options_plot

    # Retain selected values in drop_cage_plot if still valid after updating options
    drop_cage_plot.value = list(set(drop_cage_plot.value)
                                & set(cage_options_plot))

    # # Update drop_night_plot options with unique values from the "Night-Phase" column for filtered data
    # night_options_plot = filtered_df['Night-Phase'].unique()
    # drop_night_plot.options = night_options_plot
    #
    # # Retain selected values in drop_night_plot if still valid after updating options
    # drop_night_plot.value = list(set(drop_night_plot.value)
    #                              & set(night_options_plot))


# Assign the update_options function to the "observe" event of drop_injection_plot
drop_injection_plot.observe(update_options_plot, names='value')


def update_options_stat(change):
    # Filter DataFrame based on selected values in drop_injection_stat
    filtered_df = df[df['Injection'].isin(drop_injection_stat.value)]

    # Update drop_cage_stat options with unique values from the "Cage" column for filtered data
    cage_options_stats = filtered_df['Cage'].unique()
    drop_cage_stat.options = cage_options_stats

    # Retain selected values in drop_cage_stat if still valid after updating options
    drop_cage_stat.value = list(set(drop_cage_stat.value)
                                & set(cage_options_stats))

    # # Update drop_night_stat options with unique values from the "Night-Phase" column for filtered data
    # night_options_stats = filtered_df['Night-Phase'].unique()
    # drop_night_stat.options = night_options_stats
    #
    # # Retain selected values in drop_night_stat if still valid after updating options
    # drop_night_stat.value = list(set(drop_night_stat.value)
    #                              & set(night_options_stats))


# Assign the update_options function to the "observe" event of drop_injection_stat
drop_injection_stat.observe(update_options_stat, names='value')


def update_temp_df(change):
    # Update temp_df based on the current widget selection
    global temp_df
    temp_df = df[(df["Injection"].isin(drop_injection_plot.value))
                 & (df["Cage"].isin(drop_cage_plot.value))
                 & (df["name"] == drop_event.value)]
                 # & (df["Night-Phase"].isin(drop_night_plot.value))]


# Initialize temp_df based on the initial widget selection
temp_df = df[(df["Injection"].isin(drop_injection_plot.value))
             & (df["Cage"].isin(drop_cage_plot.value))
             & (df["name"] == drop_event.value)]
             # & (df["Night-Phase"].isin(drop_night_plot.value))]

# Observe widgets to update temp_df when the selection is changed
drop_injection_plot.observe(update_temp_df, 'value')
drop_cage_plot.observe(update_temp_df, 'value')
drop_event.observe(update_temp_df, 'value')
# drop_night_plot.observe(update_temp_df, 'value')

# colors for plots
colors = {j: sns.color_palette("husl", len(df.GenoA.unique()))[i] for i, j in enumerate(sorted(df.GenoA.unique()))}

# Fonction pour afficher le plot en fonction des choix de l'utilisateur
def update_dropdown(date, cage, event, range_slide_plot):
    '''
    This function will compute the 'number of event' or the 'event duration' per bin for each mice in the cage
    '''
    clear_output()
    min_value_plot, max_value_plot = range_slide_plot
    sns.set(font_scale=1.2, style='ticks')
    if choicetype.value == 'Number of events':
        for date_val, cage_val, night_phase_val in itertools.product(date, cage, night_phase):
            display(Markdown(
                f"""<h3>LMT session = {date_val}<br>Cage = {cage_val} <br>Event = {drop_event.value}
                    <br>Phase = {night_phase_val} </h3>"""))
            temp_df_loop0 = df[(df["Injection"] == date_val)
                               & (df["Cage"] == cage_val)
                               & (df["name"] == drop_event.value)
                               & (df["Night-Phase"] == night_phase_val)]
            # Créer un nouveau dataframe avec toutes les colonnes de l'ancien dataframe
            df_new0 = temp_df_loop0.copy()
            # Ajouter une colonne avec la somme des numberOfEvents de chaque RFidA de chaque Bin
            df_new0['new_numbersOfEvents'] = df_new0.groupby(['Bin', 'RFidA'])['numberOfEvents'].transform('sum')
            # Supprimer les lignes utilisées pour faire la somme
            df_new0 = df_new0.drop_duplicates(subset=['Bin', 'RFidA'],
                                            keep='last')
            df_new0 = df_new0[(df_new0['Bin'] >= min_value_plot) & (df_new0['Bin'] <= max_value_plot)]
            fig, ax1 = plt.subplots(figsize=(18, 12))
            ax2 = fig.add_axes([1, 0.25, 0.25, 0.5])
            sns.lineplot(data=df_new0,
                         y='new_numbersOfEvents',
                         x='Bin',
                         hue="GenoA",
                         style='RFidA',
                         palette=colors,
                         ax=ax1)
            ax1.set_title(f'Numbers of {drop_event.value} per bin of each mouse')
            ax1.set_ylabel(f'Numbers of {drop_event.value}')
            sns.barplot(data=df_new0,
                        y='new_numbersOfEvents',
                        x="GenoA",
                        palette=colors,
                        ax=ax2)
            ax2.set_title("Barplot which shows the number of events")
            plt.show()
    elif choicetype.value == 'Event duration':
        for date_val, cage_val, night_phase_val in itertools.product(date, cage, night_phase):
            display(Markdown(
                f"""<h3>LMT session = {date_val}<br>Cage = {cage_val} <br>Event = {drop_event.value}
                    <br>Phase = {night_phase_val} </h3>"""))
            temp_df_loop0 = df[(df["Injection"] == date_val)
                               & (df["Cage"] == cage_val)
                               & (df["name"] == drop_event.value)
                               & (df["Night-Phase"] == night_phase_val)]
            # Créer un nouveau dataframe avec toutes les colonnes de l'ancien dataframe
            df_new0 = temp_df_loop0.copy()
            # Ajouter une colonne avec la somme des numberOfEvents de chaque RFidA de chaque Bin
            df_new0['new_totallength'] = df_new0.groupby(['Bin', 'RFidA'])['totalLength'].transform('sum')
            # Supprimer les lignes utilisées pour faire la somme
            df_new0 = df_new0.drop_duplicates(subset=['Bin', 'RFidA'],
                                            keep='last')
            df_new0 = df_new0[(df_new0['Bin'] >= min_value_plot) & (df_new0['Bin'] <= max_value_plot)]
            fig, ax1 = plt.subplots(figsize=(18, 12))
            ax2 = fig.add_axes([1, 0.25, 0.25, 0.5])
            sns.lineplot(data=df_new0,
                         y='new_totallength',
                         x='Bin',
                         hue="GenoA",
                         style='RFidA',
                         palette=colors,
                         ax=ax1)
            ax1.set_title(f'Duration of {drop_event.value} per bin of each mouse')
            ax1.set_ylabel(f'Duration of {drop_event.value}')
            sns.barplot(data=df_new0,
                        y='new_totallength',
                        x="GenoA",
                        palette=colors,
                        ax=ax2)
            ax2.set_title("Barplot which shows the duration of events")
            plt.show()

# Fonction pour afficher le plot en fonction des choix de l'utilisateur
def results_update_dropdown(date, cage, event, range_slide_plot):
    '''
    This function will compute the 'number of event' or the 'event duration' per bin for each mice in the cage
    '''
    # Créer un nouveau dossier pour les fichiers
    new_folder_path = str(nom_dossier_plots.value)
    if not os.path.exists(new_folder_path):
        os.makedirs(new_folder_path)
    for r_drop_event_plot in drop_event.options:
        clear_output()
        min_value_plot, max_value_plot = range_slide_plot
        sns.set(font_scale=1.2, style='ticks')
        if choicetype.value == 'Number of events':
            for idx, (date_val, cage_val, night_phase_val) in enumerate(itertools.product(date, cage, night_phase)):
                display(Markdown(
                    f"""<h3>LMT session = {date_val}<br>Cage = {cage_val} <br>Event = {drop_event.value}
                    <br>Phase = {night_phase_val} </h3>"""))
                temp_df_loop0 = df[(df["Injection"] == date_val)
                                   & (df["Cage"] == cage_val)
                                   & (df["name"] == r_drop_event_plot)
                                   & (df["Night-Phase"] == night_phase_val)]
                # Créer un nouveau dataframe avec toutes les colonnes de l'ancien dataframe
                df_new0 = temp_df_loop0.copy()
                # Ajouter une colonne avec la somme des numberOfEvents de chaque RFidA de chaque Bin
                df_new0['new_numbersOfEvents'] = df_new0.groupby(['Bin', 'RFidA'])['numberOfEvents'].transform('sum')
                # Supprimer les lignes utilisées pour faire la somme
                df_new0 = df_new0.drop_duplicates(subset=['Bin', 'RFidA'],
                                                keep='last')
                df_new0 = df_new0[(df_new0['Bin'] >= min_value_plot) & (df_new0['Bin'] <= max_value_plot)]
                fig, ax1 = plt.subplots(figsize=(18, 12))
                ax2 = fig.add_axes([1, 0.25, 0.25, 0.5])
                sns.lineplot(data=df_new0,
                             y='new_numbersOfEvents',
                             x='Bin',
                             hue="GenoA",
                             style='RFidA',
                             palette=colors,
                             ax=ax1)
                ax1.set_title(f'Numbers of {r_drop_event_plot} per bin of each mouse of {date_val} in {cage_val}')
                ax1.set_ylabel(f'Numbers of {r_drop_event_plot}')
                sns.barplot(data=df_new0,
                            y='new_numbersOfEvents',
                            x="GenoA",
                            palette=colors,
                            ax=ax2)
                ax2.set_title("Barplot which shows the number of events")
                image_file_path = os.path.join(new_folder_path, f'NumberofEvent_{date_val}_{cage_val}_night{night_phase_val}_{r_drop_event_plot}.png')
                plt.savefig(image_file_path,
                            dpi=300,
                            bbox_inches='tight')
                plt.close(fig)
        elif choicetype.value == 'Event duration':
            for idx, (date_val, cage_val, night_phase_val) in enumerate(itertools.product(date, cage, night_phase)):
                display(Markdown(
                    f"""<h3>LMT session = {date_val}<br>Cage = {cage_val} <br>Event = {drop_event.value}
                    <br>Phase = {night_phase_val} </h3>"""))
                temp_df_loop0 = df[(df["Injection"] == date_val)
                                   & (df["Cage"] == cage_val)
                                   & (df["name"] == r_drop_event_plot)
                                   & (df["Night-Phase"] == night_phase_val)]
                # Créer un nouveau dataframe avec toutes les colonnes de l'ancien dataframe
                df_new0 = temp_df_loop0.copy()
                # Ajouter une colonne avec la somme des numberOfEvents de chaque RFidA de chaque Bin
                df_new0['new_totallength'] = df_new0.groupby(['Bin', 'RFidA'])['totalLength'].transform('sum')
                # Supprimer les lignes utilisées pour faire la somme
                df_new0 = df_new0.drop_duplicates(subset=['Bin', 'RFidA'],
                                                keep='last')
                df_new0 = df_new0[(df_new0['Bin'] >= min_value_plot) & (df_new0['Bin'] <= max_value_plot)]
                fig, ax1 = plt.subplots(figsize=(18, 12))
                ax2 = fig.add_axes([1, 0.25, 0.25, 0.5])
                sns.lineplot(data=df_new0,
                             y='new_totallength',
                             x='Bin',
                             hue="GenoA",
                             style='RFidA',
                             palette=colors,
                             ax=ax1)
                ax1.set_title(f'Duration of {r_drop_event_plot} per bin of each mouse of {date_val} in {cage_val}')
                ax1.set_ylabel(f'Duration of {r_drop_event_plot}')
                sns.barplot(data=df_new0,
                            y='new_totallength',
                            x="GenoA",
                            palette=colors,
                            ax=ax2)
                ax2.set_title("Barplot which shows the duration of events")
                image_file_path = os.path.join(new_folder_path, f'Eventduration_{date_val}_{cage_val}_night{night_phase_val}_{r_drop_event_plot}.png')
                plt.savefig(image_file_path,
                            dpi=300,
                            bbox_inches='tight')
                plt.close(fig)
        else :
            return


def update_stats(date, genos, cage, event, choice_type, range_slide_stats):
    '''
    This function will return a figure with 3 plots:
    . The first with the dabest package that will compare the selected data (eg. NaCl/Amphet)
    . The second with the dabest package that will compare the different injections
    . The third that is the summary of the statsmodels.stats.descriptivestats.describe() function
    '''
    global pivot_df2
    global merged_df
    global selected_genos
    global temp_df_loop1
    global image_file_path
    clear_output()
    new_column_names = []
    dfs = []
    ids = [i for i in df.GenoA.unique()]
    min_value_stats, max_value_stats = range_slide_stats

    if choicetype.value == 'Number of events':
        # temp_df_loop1 = df[(df["GenoA"].isin(list(genos.value)))]
        for date_val, genos_val, cage_val, night_phase_val in itertools.product(date, genos, cage, night_phase):
            # display(Markdown(f"""<h3>L'analyse est réalisée sur {animalnumber.value[0]} souris, l'injection
            # choisie est '{date_val}',
            # il s'agit de la {cage_val} pour l'évènement '{drop_event.value}' durant la phase {night_phase_val}
            # de nuit !</h3>"""))
            temp_df_loop1 = df[(df["Injection"] == date_val)
                               & (df["GenoA"] == genos_val)
                               & (df["Cage"] == cage_val)
                               & (df["name"] == drop_event.value)
                               & (df["Night-Phase"] == night_phase_val)]
            # Créer un nouveau dataframe avec toutes les colonnes de l'ancien dataframe
            df_new = temp_df_loop1.copy()
            # df_new = df_new[df_new['GenoA'].isin(selected_genos)]
            # Ajouter une colonne avec la somme des numberOfEvents de chaque RFidA de chaque Bin
            df_new['new_numbersOfEvents'] = df_new.groupby(['Bin', 'RFidA'])['numberOfEvents'].transform('sum')
            # Supprimer les lignes utilisées pour faire la somme
            df_new = df_new.drop_duplicates(subset=['Bin', 'RFidA'],
                                            keep='last')
            df_new = df_new[(df_new['Bin'] >= min_value_stats) & (df_new['Bin'] <= max_value_stats)]
            # fig, ax1 = plt.subplots(figsize=(10,4))
            # ax2 = fig.add_axes([1, 0.25, 0.25, 0.5])
            # sns.barplot(data=df_new, y='new_numbersOfEvents', x="GenoA", palette={'Amphet':'red', 'NaCl':'blue'}, ax=ax2)
            # ax2.set_title("A barplot")
            # plt.show()
            dfs.append(df_new)

        if not dfs:
            return

        else:
            merged_df = pd.concat(dfs)
            merged_df['new_cage'] = merged_df['Cage'].str.extract('(\d+)').astype(int)
            global df_lm
            global result_df
            global two_groups_paired_baseline
            global column_index
            df_lm = merged_df.copy()
            df_lm = merged_df.groupby(['new_cage', 'RFidA', 'GenoA', 'Date'])['new_numbersOfEvents'].sum().reset_index()

            # Compute Linear Mixed Models stats

            if drop_stat.value == "Linear Mixed Model":
                ssdd = statsmodels.stats.descriptivestats.describe(df_lm,
                                                                   stats=['mean', 'std_err', 'std', 'median'])
                model = smf.mixedlm("new_numbersOfEvents ~ GenoA",
                                    df_lm,
                                    groups='new_cage')  # Creates the model
                result = model.fit()  # Run model

                # df_lm['Dates'] = df_lm.groupby('RFidA')['Date'].transform(lambda x: ', '.join(x.astype(str)))
                # Utilisez groupby et cumcount pour attribuer des valeurs "1", "2" ou "3" en fonction de l'ordre des dates pour chaque RFidA
                # df_lm['New_Column'] = df_lm.groupby('RFidA').cumcount() + 1
                # df_lm['New_Column'] = df_lm['New_Column'].astype(str)
                df_lm = df_lm.sort_values(by=['RFidA', 'new_cage', 'Date']).reset_index(drop=True)

                new_df = df_lm.loc[:, ['new_cage', 'RFidA']]

                # Récupérez les valeurs de sélections dans le widget
                name_selections = list(drop_injection_stat.value)

                num_select = len(name_selections)

                # Ajoutez une colonne 'Injection' au DataFrame
                df_lm['Injection'] = [name_selections[i % num_select] for i in range(len(df_lm))]

                # Créez un dictionnaire pour stocker les données pivotées
                pivot_data = {'new_cage': [], 'RFidA': []}
                for injection in name_selections:
                    pivot_data[injection] = []

                # Parcourez les lignes du premier DataFrame
                for _, row in df_lm.iterrows():
                    liste_newcage = row['new_cage']
                    liste_RFidA = row['RFidA']
                    liste_injection = row['Injection']
                    liste_nbevents = row['new_numbersOfEvents']

                    # Vérifiez si la valeur de 'Injection' est dans la liste sélectionnée
                    if liste_injection in name_selections:
                        pivot_data['new_cage'].append(liste_newcage)
                        pivot_data['RFidA'].append(liste_RFidA)
                        for selected_injection in name_selections:
                            if selected_injection == liste_injection:
                                pivot_data[selected_injection].append(liste_nbevents)
                            else:
                                pivot_data[selected_injection].append(None)

                # Créez un DataFrame à partir du dictionnaire pivoté
                pivot_df = pd.DataFrame(pivot_data)

                # Fusionnez df2 avec le DataFrame pivoté
                result_df = pd.merge(new_df, pivot_df, on=['new_cage', 'RFidA'], how='left')

                # Groupez le résultat par 'new_cage' et 'RFidA' et agrégez les valeurs
                result_df = result_df.groupby(['new_cage', 'RFidA'])[name_selections].first().reset_index()

                # Replace values NaN by 'None'
                result_df = result_df.fillna('0')
                result_df.reset_index(drop=True, inplace=True)
                result_df.insert(0, 'Index', range(len(result_df)))

                # Reinitialize the index of the 2nd df
                new_df = new_df.reset_index(drop=True)

                # Convert all columns except 'Index', 'new_cage' and 'RFidA' in int64
                result_df[result_df.columns.difference(['Index', 'new_cage', 'RFidA'])] = result_df[
                    result_df.columns.difference(['Index', 'new_cage', 'RFidA'])].astype('int64')
                result_df
                pivot_df2 = df_lm.pivot(index=['RFidA', 'new_cage'],
                                        columns='Date',
                                        values='new_numbersOfEvents').reset_index()
                pivot_df2.sort_values('new_cage',
                                      inplace=True)

                # Initializing for dabest
                two_groups_paired_baseline = dabest.load(data=result_df,
                                                         idx=list(drop_injection_stat.value),
                                                         id_col="Index",
                                                         paired='baseline')

                fig, axs = plt.subplots(1, 3,
                                        figsize=(30, 12))
                fig.subplots_adjust(wspace=0.8)

                # 1er plot
                plotdabest = dabest.load(data=df_lm,
                                         x="GenoA",
                                         y="new_numbersOfEvents",
                                         idx=list(genos),
                                         id_col="new_cage")

                # Tracer le graphique
                plotdabest.mean_diff.plot(ax=axs[0],
                                          color_col='new_cage')
                axs[0].set_title(f"""Sum of numberOfEvents per Cage for the '{drop_event.value}' event""",
                                 loc='left')

                # 2ème plot
                if len(drop_injection_stat.value) == 2:
                    ax1 = two_groups_paired_baseline.mean_diff.plot(color_col='new_cage',
                                                                    ax=axs[1])
                    axs[1].legend(title='Cage',
                                  labels=result_df['new_cage'].unique(),
                                  frameon=False,
                                  loc=(1.62, 0.75))
                elif len(drop_injection_stat.value) > 2:
                    ax1 = two_groups_paired_baseline.mean_diff.plot(color_col='new_cage',
                                                                    ax=axs[1])
                    axs[1].legend(title='Cage',
                                  labels=result_df['new_cage'].unique(),
                                  frameon=False,
                                  loc=(0.95, 0.75))

                # 3ème plot
                axs[2].axis('off')
                table0 = tabulate(ssdd,
                                  headers="keys",
                                  colalign=("center", "center", "center", "center", "center"))
                axs[2].annotate(table0,
                                xy=(-0.45, 0.75),
                                xycoords='axes fraction',
                                fontsize=10, va='top',
                                family='monospace')
                table1 = tabulate(result.summary().tables[0],
                                  colalign=("center", "center", "center", "center", "center"))
                axs[2].annotate(table1,
                                xy=(-0.45, 0.55),
                                xycoords='axes fraction',
                                fontsize=10,
                                va='top',
                                family='monospace')
                table2 = tabulate(result.summary().tables[1],
                                  headers="keys",
                                  tablefmt="plain",
                                  colalign=("center", "center", "center", "center", "center", "center", "center"))
                axs[2].annotate(table2,
                                xy=(-0.45, 0.35),
                                xycoords='axes fraction',
                                fontsize=10,
                                va='top',
                                family='monospace')

                df_lm = df_lm.rename(columns={'new_cage': 'Cage', 'new_numbersOfEvents': 'NumberOfEvents'})

    #             elif drop_stat.value == "ANOVA":
    #                 # 1
    #                 # model = smf.ols("new_numbersOfEvents ~ GenoA", data=df_lm)
    #                 # anova_result = sm.stats.anova_lm(model.fit())
    #                 # print(anova_result)
    #                 # 2
    #                 anova_result = smf.ols("new_numbersOfEvents ~ GenoA", data=df_lm).fit()
    #                 # print(anova_result.summary())
    #                 pivot_df2 = df_lm.pivot(index=['RFidA', 'new_cage'],
    #                                         columns='Date',
    #                                         values='new_numbersOfEvents').reset_index()
    #                 pivot_df2.sort_values('new_cage',
    #                                       inplace=True)

    #                 # Récupérez les valeurs de sélections dans le widget
    #                 name_selections = list(drop_injection_stat.value)

    #                 ##### MARCHE SI LES VALEURS SONT COLLéES (PAR DE NAN ENTRE 2 VALEURS)
    #                 # Remplissez les colonnes avec la somme des colonnes existantes à partir de la deuxième colonne
    #                 for i in range(1, num_selections + 1):
    #                     column_name = name_selections[i-1]
    #                     start_column = i + 1  # Commencez à partir de la colonne suivante (index + 1)
    #                     step = num_selections  # Utilisez un pas égal au nombre de sélections
    #                     pivot_df2[column_name] = pivot_df2.iloc[:, start_column::step].sum(axis=1)
    #                     new_column_names.append(column_name)  # Ajoutez le nom de la colonne à la liste

    #                 # création de la colonne avec l'index
    #                 pivot_df2.reset_index(inplace=True)
    #                 pivot_df2.index = range(len(pivot_df2))
    #                 pivot_df2.index.name = 'Index'
    #                 pivot_df2['Index'] = pivot_df2.index.astype(int)

    #                 two_groups_paired_baseline = dabest.load(data=pivot_df2,
    #                                                              idx=new_column_names,
    #                                                              id_col="Index",
    #                                                              paired='baseline')

    #                 fig, axs = plt.subplots(1, 3,
    #                                         figsize=(30, 12))
    #                 fig.subplots_adjust(wspace=0.8)

    #                 # 1er plot
    #                 plotdabest = dabest.load(data=df_lm,
    #                                          x="GenoA",
    #                                          y="new_numbersOfEvents",
    #                                          idx=ids,
    #                                          id_col="new_cage")

    #                 # Tracer le graphique
    #                 plotdabest.mean_diff.plot(ax=axs[0],
    #                                           color_col='new_cage')
    #                 axs[0].set_title(f"""Sum of numberOfEvents per Cage for the '{drop_event.value}' event""",
    #                                  loc='left')

    #                 # 2ème plot
    #                 if len(drop_injection_stat.value) == 2:
    #                     ax1 = two_groups_paired_baseline.mean_diff.plot(color_col='new_cage',
    #                                                                     ax=axs[1])
    #                     axs[1].legend(title='Cage',
    #                                   labels=pivot_df2['new_cage'].unique(),
    #                                   frameon=False,
    #                                   loc=(1.62, 0.75))
    #                 elif len(drop_injection_stat.value) > 2:
    #                     ax1 = two_groups_paired_baseline.mean_diff.plot(color_col='new_cage',
    #                                                                     ax=axs[1])
    #                     axs[1].legend(title='Cage',
    #                                   labels=pivot_df2['new_cage'].unique(),
    #                                   frameon=False,
    #                                   loc=(0.95, 0.75))

    #                 # 3ème plot
    #                 axs[2].set_position([0.7, 0.1, 0.4, 0.8])  # Ajuster la position et la taille
    #                 axs[2].axis('off')
    #                 # Annotation de texte pour le modèle ajusté
    #                 axs[2].text(0.5, 0.5, str(anova_result.summary()), fontsize=12, family='monospace', ha='right', va='center')
    #                 # axs[2].text(0.1, 0.5, str(anova_result.summary()), fontsize=10, family='monospace')
    #                 # table0 = tabulate(anova_result,
    #                 #                   headers="keys",
    #                 #                   colalign=("center", "center", "center", "center", "center"))
    #                 # axs[2].annotate(table0,
    #                 #                 xy=(-0.25, 0.75),
    #                 #                 xycoords='axes fraction',
    #                 #                 fontsize=10, va='top',
    #                 #                 family='monospace')
    #                 # table1 = tabulate(anova_result.summary().tables[0],
    #                 #                   colalign=("center", "center", "center", "center", "center"))
    #                 # axs[2].annotate(table1,
    #                 #                 xy=(-0.25, 0.55),
    #                 #                 xycoords='axes fraction',
    #                 #                 fontsize=10,
    #                 #                 va='top',
    #                 #                 family='monospace')
    #                 # table2 = tabulate(anova_result.summary().tables[1],
    #                 #                   headers="keys",
    #                 #                   tablefmt="plain",
    #                 #                   colalign=("center", "center", "center", "center", "center", "center", "center"))
    #                 # axs[2].annotate(table2,
    #                 #                 xy=(-0.25, 0.35),
    #                 #                 xycoords='axes fraction',
    #                 #                 fontsize=10,
    #                 #                 va='top',
    #                 #                 family='monospace')

    elif choicetype.value == 'Event duration':
        for date_val, genos_val, cage_val, night_phase_val in itertools.product(date, genos, cage, night_phase):
            temp_df_loop1 = df[(df["Injection"] == date_val)
                               & (df["GenoA"] == genos_val)
                               & (df["Cage"] == cage_val)
                               & (df["name"] == drop_event.value)
                               & (df["Night-Phase"] == night_phase_val)]
            # Créer un nouveau dataframe avec toutes les colonnes de l'ancien dataframe
            df_new = temp_df_loop1.copy()
            # Ajouter une colonne avec la somme des numberOfEvents de chaque RFidA de chaque Bin
            df_new['new_totallength'] = df_new.groupby(['Bin', 'RFidA'])['totalLength'].transform('sum')
            # Supprimer les lignes utilisées pour faire la somme
            df_new = df_new.drop_duplicates(subset=['Bin', 'RFidA'],
                                            keep='last')
            df_new = df_new[(df_new['Bin'] >= min_value_stats) & (df_new['Bin'] <= max_value_stats)]
            dfs.append(df_new)

        if not dfs:
            return

        else:
            merged_df = pd.concat(dfs)
            merged_df['new_cage'] = merged_df['Cage'].str.extract('(\d+)').astype(int)
            df_lm = merged_df.copy()
            df_lm = merged_df.groupby(['new_cage', 'RFidA', 'GenoA', 'Date'])['new_totallength'].sum().reset_index()

            if drop_stat.value == "Linear Mixed Model":
                ssdd = statsmodels.stats.descriptivestats.describe(df_lm,
                                                                   stats=['mean', 'std_err', 'std', 'median'])
                model = smf.mixedlm("new_totallength ~ GenoA",
                                    df_lm,
                                    groups='new_cage')  # Creates the model
                result = model.fit()  # Run model

                df_lm = df_lm.sort_values(by=['RFidA', 'new_cage', 'Date']).reset_index(drop=True)

                new_df = df_lm.loc[:, ['new_cage', 'RFidA']]

                # Récupérez les valeurs de sélections dans le widget
                name_selections = list(drop_injection_stat.value)

                num_select = len(name_selections)

                # Ajoutez une colonne 'Injection' au DataFrame
                df_lm['Injection'] = [name_selections[i % num_select] for i in range(len(df_lm))]

                # Créez un dictionnaire pour stocker les données pivotées
                pivot_data = {'new_cage': [], 'RFidA': []}
                for injection in name_selections:
                    pivot_data[injection] = []

                # Parcourez les lignes du premier DataFrame
                for _, row in df_lm.iterrows():
                    liste_newcage = row['new_cage']
                    liste_RFidA = row['RFidA']
                    liste_injection = row['Injection']
                    liste_nbevents = row['new_totallength']

                    # Vérifiez si la valeur de 'Injection' est dans la liste sélectionnée
                    if liste_injection in name_selections:
                        pivot_data['new_cage'].append(liste_newcage)
                        pivot_data['RFidA'].append(liste_RFidA)
                        for selected_injection in name_selections:
                            if selected_injection == liste_injection:
                                pivot_data[selected_injection].append(liste_nbevents)
                            else:
                                pivot_data[selected_injection].append(None)

                # Créez un DataFrame à partir du dictionnaire pivoté
                pivot_df = pd.DataFrame(pivot_data)

                # Fusionnez df2 avec le DataFrame pivoté
                result_df = pd.merge(new_df, pivot_df, on=['new_cage', 'RFidA'], how='left')

                # Groupez le résultat par 'new_cage' et 'RFidA' et agrégez les valeurs
                result_df = result_df.groupby(['new_cage', 'RFidA'])[name_selections].first().reset_index()

                # Remplacez les valeurs NaN par 'None'
                result_df = result_df.fillna('0')
                result_df.reset_index(drop=True, inplace=True)
                result_df.insert(0, 'Index', range(len(result_df)))

                # Réinitialisez l'index du deuxième DataFrame
                new_df = new_df.reset_index(drop=True)

                # Convertir toutes les colonnes sauf 'Index', 'new_cage', et 'RFidA' en int64
                result_df[result_df.columns.difference(['Index', 'new_cage', 'RFidA'])] = result_df[
                    result_df.columns.difference(['Index', 'new_cage', 'RFidA'])].astype('int64')
                result_df
                pivot_df2 = df_lm.pivot(index=['RFidA', 'new_cage'],
                                        columns='Date',
                                        values='new_totallength').reset_index()
                pivot_df2.sort_values('new_cage',
                                      inplace=True)

                two_groups_paired_baseline = dabest.load(data=result_df,
                                                         idx=list(drop_injection_stat.value),
                                                         id_col="Index",
                                                         paired='baseline')

                fig, axs = plt.subplots(1, 3,
                                        figsize=(30, 12))
                fig.subplots_adjust(wspace=0.8)

                # 1er plot
                plotdabest = dabest.load(data=df_lm,
                                         x="GenoA",
                                         y="new_totallength",
                                         idx=list(genos),
                                         id_col="new_cage")

                # Tracer le graphique
                plotdabest.mean_diff.plot(ax=axs[0],
                                          color_col='new_cage')
                axs[0].set_title(f"""Sum of totalLength per Cage for the '{drop_event.value}' event""",
                                 loc='left')

                # 2ème plot
                if len(drop_injection_stat.value) == 2:
                    ax1 = two_groups_paired_baseline.mean_diff.plot(color_col='new_cage',
                                                                    ax=axs[1])
                    axs[1].legend(title='Cage',
                                  labels=result_df['new_cage'].unique(),
                                  frameon=False,
                                  loc=(1.62, 0.75))
                elif len(drop_injection_stat.value) > 2:
                    ax1 = two_groups_paired_baseline.mean_diff.plot(color_col='new_cage',
                                                                    ax=axs[1])
                    axs[1].legend(title='Cage',
                                  labels=result_df['new_cage'].unique(),
                                  frameon=False,
                                  loc=(0.95, 0.75))

                # 3ème plot
                axs[2].axis('off')
                table0 = tabulate(ssdd,
                                  headers="keys",
                                  colalign=("center", "center", "center", "center", "center"))
                axs[2].annotate(table0,
                                xy=(-0.45, 0.75),
                                xycoords='axes fraction',
                                fontsize=10, va='top',
                                family='monospace')
                table1 = tabulate(result.summary().tables[0],
                                  colalign=("center", "center", "center", "center", "center"))
                axs[2].annotate(table1,
                                xy=(-0.45, 0.55),
                                xycoords='axes fraction',
                                fontsize=10,
                                va='top',
                                family='monospace')
                table2 = tabulate(result.summary().tables[1],
                                  headers="keys",
                                  tablefmt="plain",
                                  colalign=("center", "center", "center", "center", "center", "center", "center"))
                axs[2].annotate(table2,
                                xy=(-0.45, 0.35),
                                xycoords='axes fraction',
                                fontsize=10,
                                va='top',
                                family='monospace')

                df_lm = df_lm.rename(columns={'new_cage': 'Cage', 'new_totallength': 'Totallength'})

def results_update_stats(date, genos, cage, event, choice_type, range_slide_stats):
    '''
    This function will return a figure with 3 plots:
    . The first with the dabest package that will compare the selected data (eg. NaCl/Amphet)
    . The second with the dabest package that will compare the different injections
    . The third that is the summary of the statsmodels.stats.descriptivestats.describe() function
    '''
    global pivot_df2
    global merged_df
    global selected_genos
    global temp_df_loop1
    global df_new
    clear_output()
    new_column_names = []
    dfs = []
    ids = [i for i in df.GenoA.unique()]
    min_value_stats, max_value_stats = range_slide_stats
    if drop_stat.value == "Linear Mixed Model":
        # Créer un nouveau dossier pour les fichiers
        new_folder_path = str(nom_dossier_stats.value)
        if not os.path.exists(new_folder_path):
            os.makedirs(new_folder_path)
        if choicetype.value == 'Number of events':
            # temp_df_loop1 = df[(df["GenoA"].isin(list(genos.value)))]
            for date_val, genos_val, cage_val, night_phase_val in itertools.product(date, genos, cage, night_phase):
                # display(Markdown(f"""<h3>L'analyse est réalisée sur {animalnumber.value[0]} souris, l'injection
                # choisie est '{date_val}',
                # il s'agit de la {cage_val} pour l'évènement '{drop_event.value}' durant la phase {night_phase_val}
                # de nuit !</h3>"""))
                temp_df_loop1 = df[(df["Injection"] == date_val)
                                   & (df["GenoA"] == genos_val)
                                   & (df["Cage"] == cage_val)
                                   & (df["name"] == drop_event.value)
                                   & (df["Night-Phase"] == night_phase_val)]
                # Créer un nouveau dataframe avec toutes les colonnes de l'ancien dataframe
                df_new = temp_df_loop1.copy()
                # df_new = df_new[df_new['GenoA'].isin(selected_genos)]
                # Ajouter une colonne avec la somme des numberOfEvents de chaque RFidA de chaque Bin
                df_new['new_numbersOfEvents'] = df_new.groupby(['Bin', 'RFidA'])['numberOfEvents'].transform('sum')
                # Supprimer les lignes utilisées pour faire la somme
                df_new = df_new.drop_duplicates(subset=['Bin', 'RFidA'],
                                                keep='last')
                df_new = df_new[(df_new['Bin'] >= min_value_stats) & (df_new['Bin'] <= max_value_stats)]
                # fig, ax1 = plt.subplots(figsize=(10,4))
                # ax2 = fig.add_axes([1, 0.25, 0.25, 0.5])
                # sns.barplot(data=df_new, y='new_numbersOfEvents', x="GenoA", palette={'Amphet':'red', 'NaCl':'blue'}, ax=ax2)
                # ax2.set_title("A barplot")
                # plt.show()
                dfs.append(df_new)

            if not dfs:
                return

            else:
                merged_df = pd.concat(dfs)
                merged_df['new_cage'] = merged_df['Cage'].str.extract('(\d+)').astype(int)
                global df_lm
                global result_df
                global two_groups_paired_baseline
                global column_index
                df_lm = merged_df.copy()
                df_lm = merged_df.groupby(['new_cage', 'RFidA', 'GenoA', 'Date'])['new_numbersOfEvents'].sum().reset_index()

                # Compute Linear Mixed Models stats

                if drop_stat.value == "Linear Mixed Model":
                    ssdd = statsmodels.stats.descriptivestats.describe(df_lm,
                                                                       stats=['mean', 'std_err', 'std', 'median'])
                    model = smf.mixedlm("new_numbersOfEvents ~ GenoA",
                                        df_lm,
                                        groups='new_cage')  # Creates the model
                    result = model.fit()  # Run model

                    # df_lm['Dates'] = df_lm.groupby('RFidA')['Date'].transform(lambda x: ', '.join(x.astype(str)))
                    # Utilisez groupby et cumcount pour attribuer des valeurs "1", "2" ou "3" en fonction de l'ordre des dates pour chaque RFidA
                    # df_lm['New_Column'] = df_lm.groupby('RFidA').cumcount() + 1
                    # df_lm['New_Column'] = df_lm['New_Column'].astype(str)
                    df_lm = df_lm.sort_values(by=['RFidA', 'new_cage', 'Date']).reset_index(drop=True)

                    new_df = df_lm.loc[:, ['new_cage', 'RFidA']]

                    # Récupérez les valeurs de sélections dans le widget
                    name_selections = list(drop_injection_stat.value)

                    num_select = len(name_selections)

                    # Ajoutez une colonne 'Injection' au DataFrame
                    df_lm['Injection'] = [name_selections[i % num_select] for i in range(len(df_lm))]

                    # Créez un dictionnaire pour stocker les données pivotées
                    pivot_data = {'new_cage': [], 'RFidA': []}
                    for injection in name_selections:
                        pivot_data[injection] = []

                    # Parcourez les lignes du premier DataFrame
                    for _, row in df_lm.iterrows():
                        liste_newcage = row['new_cage']
                        liste_RFidA = row['RFidA']
                        liste_injection = row['Injection']
                        liste_nbevents = row['new_numbersOfEvents']

                        # Vérifiez si la valeur de 'Injection' est dans la liste sélectionnée
                        if liste_injection in name_selections:
                            pivot_data['new_cage'].append(liste_newcage)
                            pivot_data['RFidA'].append(liste_RFidA)
                            for selected_injection in name_selections:
                                if selected_injection == liste_injection:
                                    pivot_data[selected_injection].append(liste_nbevents)
                                else:
                                    pivot_data[selected_injection].append(None)

                    # Créez un DataFrame à partir du dictionnaire pivoté
                    pivot_df = pd.DataFrame(pivot_data)

                    # Fusionnez df2 avec le DataFrame pivoté
                    result_df = pd.merge(new_df, pivot_df, on=['new_cage', 'RFidA'], how='left')

                    # Groupez le résultat par 'new_cage' et 'RFidA' et agrégez les valeurs
                    result_df = result_df.groupby(['new_cage', 'RFidA'])[name_selections].first().reset_index()

                    # Replace values NaN by 'None'
                    result_df = result_df.fillna('0')
                    result_df.reset_index(drop=True, inplace=True)
                    result_df.insert(0, 'Index', range(len(result_df)))

                    # Reinitialize the index of the 2nd df
                    new_df = new_df.reset_index(drop=True)

                    # Convert all columns except 'Index', 'new_cage' and 'RFidA' in int64
                    result_df[result_df.columns.difference(['Index', 'new_cage', 'RFidA'])] = result_df[result_df.columns.difference(['Index', 'new_cage', 'RFidA'])].astype('int64')
                    result_df
                    pivot_df2 = df_lm.pivot(index=['RFidA', 'new_cage'],
                                            columns='Date',
                                            values='new_numbersOfEvents').reset_index()
                    pivot_df2.sort_values('new_cage',
                                          inplace=True)

                    # Initializing for dabest
                    two_groups_paired_baseline = dabest.load(data=result_df,
                                                                 idx=list(drop_injection_stat.value),
                                                                 id_col="Index",
                                                                 paired='baseline')

                    fig, axs = plt.subplots(1, 3,
                                            figsize=(30, 12))
                    fig.subplots_adjust(wspace=0.8)

                    # 1er plot
                    plotdabest = dabest.load(data=df_lm,
                                             x="GenoA",
                                             y="new_numbersOfEvents",
                                             idx=list(genos),
                                             id_col="new_cage")

                    # Tracer le graphique
                    plotdabest.mean_diff.plot(ax=axs[0],
                                              color_col='new_cage')
                    axs[0].set_title(f"""Sum of numberOfEvents per Cage for the '{drop_event.value}' event""",
                                     loc='left')

                    # 2ème plot
                    if len(drop_injection_stat.value) == 2:
                        ax1 = two_groups_paired_baseline.mean_diff.plot(color_col='new_cage',
                                                                        ax=axs[1])
                        axs[1].legend(title='Cage',
                                      labels=result_df['new_cage'].unique(),
                                      frameon=False,
                                      loc=(1.62, 0.75))
                    elif len(drop_injection_stat.value) > 2:
                        ax1 = two_groups_paired_baseline.mean_diff.plot(color_col='new_cage',
                                                                        ax=axs[1])
                        axs[1].legend(title='Cage',
                                      labels=result_df['new_cage'].unique(),
                                      frameon=False,
                                      loc=(0.95, 0.75))

                    # 3ème plot
                    axs[2].axis('off')
                    table0 = tabulate(ssdd,
                                      headers="keys",
                                      colalign=("center", "center", "center", "center", "center"))
                    axs[2].annotate(table0,
                                    xy=(-0.45, 0.75),
                                    xycoords='axes fraction',
                                    fontsize=10, va='top',
                                    family='monospace')
                    table1 = tabulate(result.summary().tables[0],
                                      colalign=("center", "center", "center", "center", "center"))
                    axs[2].annotate(table1,
                                    xy=(-0.45, 0.55),
                                    xycoords='axes fraction',
                                    fontsize=10,
                                    va='top',
                                    family='monospace')
                    table2 = tabulate(result.summary().tables[1],
                                      headers="keys",
                                      tablefmt="plain",
                                      colalign=("center", "center", "center", "center", "center", "center", "center"))
                    axs[2].annotate(table2,
                                    xy=(-0.45, 0.35),
                                    xycoords='axes fraction',
                                    fontsize=10,
                                    va='top',
                                    family='monospace')
                    image_file_path = os.path.join(new_folder_path, f'{drop_event.value}.png')
                    plt.savefig(image_file_path, dpi=300, bbox_inches='tight')
                    plt.close(fig)

                    df_lm = df_lm.rename(columns={'new_cage': 'Cage', 'new_numbersOfEvents': 'NumberOfEvents'})

        elif choicetype.value == 'Event duration':
            for date_val, genos_val, cage_val, night_phase_val in itertools.product(date, genos, cage, night_phase):
                temp_df_loop1 = df[(df["Injection"] == date_val)
                                   & (df["GenoA"] == genos_val)
                                   & (df["Cage"] == cage_val)
                                   & (df["name"] == drop_event.value)
                                   & (df["Night-Phase"] == night_phase_val)]
                # Créer un nouveau dataframe avec toutes les colonnes de l'ancien dataframe
                df_new = temp_df_loop1.copy()
                # Ajouter une colonne avec la somme des numberOfEvents de chaque RFidA de chaque Bin
                df_new['new_totallength'] = df_new.groupby(['Bin', 'RFidA'])['totalLength'].transform('sum')
                # Supprimer les lignes utilisées pour faire la somme
                df_new = df_new.drop_duplicates(subset=['Bin', 'RFidA'],
                                                keep='last')
                df_new = df_new[(df_new['Bin'] >= min_value_stats) & (df_new['Bin'] <= max_value_stats)]
                dfs.append(df_new)

            if not dfs:
                return

            else:
                merged_df = pd.concat(dfs)
                merged_df['new_cage'] = merged_df['Cage'].str.extract('(\d+)').astype(int)
                df_lm = merged_df.copy()
                df_lm = merged_df.groupby(['new_cage', 'RFidA', 'GenoA', 'Date'])['new_totallength'].sum().reset_index()

                if drop_stat.value == "Linear Mixed Model":
                    ssdd = statsmodels.stats.descriptivestats.describe(df_lm,
                                                                       stats=['mean', 'std_err', 'std', 'median'])
                    model = smf.mixedlm("new_totallength ~ GenoA",
                                        df_lm,
                                        groups='new_cage')  # Creates the model
                    result = model.fit()  # Run model

                    df_lm = df_lm.sort_values(by=['RFidA', 'new_cage', 'Date']).reset_index(drop=True)

                    new_df = df_lm.loc[:, ['new_cage', 'RFidA']]

                    # Récupérez les valeurs de sélections dans le widget
                    name_selections = list(drop_injection_stat.value)

                    num_select = len(name_selections)

                    # Ajoutez une colonne 'Injection' au DataFrame
                    df_lm['Injection'] = [name_selections[i % num_select] for i in range(len(df_lm))]

                    # Créez un dictionnaire pour stocker les données pivotées
                    pivot_data = {'new_cage': [], 'RFidA': []}
                    for injection in name_selections:
                        pivot_data[injection] = []

                    # Parcourez les lignes du premier DataFrame
                    for _, row in df_lm.iterrows():
                        liste_newcage = row['new_cage']
                        liste_RFidA = row['RFidA']
                        liste_injection = row['Injection']
                        liste_nbevents = row['new_totallength']

                        # Vérifiez si la valeur de 'Injection' est dans la liste sélectionnée
                        if liste_injection in name_selections:
                            pivot_data['new_cage'].append(liste_newcage)
                            pivot_data['RFidA'].append(liste_RFidA)
                            for selected_injection in name_selections:
                                if selected_injection == liste_injection:
                                    pivot_data[selected_injection].append(liste_nbevents)
                                else:
                                    pivot_data[selected_injection].append(None)

                    # Créez un DataFrame à partir du dictionnaire pivoté
                    pivot_df = pd.DataFrame(pivot_data)

                    # Fusionnez df2 avec le DataFrame pivoté
                    result_df = pd.merge(new_df, pivot_df, on=['new_cage', 'RFidA'], how='left')

                    # Groupez le résultat par 'new_cage' et 'RFidA' et agrégez les valeurs
                    result_df = result_df.groupby(['new_cage', 'RFidA'])[name_selections].first().reset_index()

                    # Remplacez les valeurs NaN par 'None'
                    result_df = result_df.fillna('0')
                    result_df.reset_index(drop=True, inplace=True)
                    result_df.insert(0, 'Index', range(len(result_df)))

                    # Réinitialisez l'index du deuxième DataFrame
                    new_df = new_df.reset_index(drop=True)

                    # Convertir toutes les colonnes sauf 'Index', 'new_cage', et 'RFidA' en int64
                    result_df[result_df.columns.difference(['Index', 'new_cage', 'RFidA'])] = result_df[result_df.columns.difference(['Index', 'new_cage', 'RFidA'])].astype('int64')
                    result_df
                    pivot_df2 = df_lm.pivot(index=['RFidA', 'new_cage'],
                                            columns='Date',
                                            values='new_totallength').reset_index()
                    pivot_df2.sort_values('new_cage',
                                          inplace=True)

                    two_groups_paired_baseline = dabest.load(data=result_df,
                                                                 idx=list(drop_injection_stat.value),
                                                                 id_col="Index",
                                                                 paired='baseline')

                    fig, axs = plt.subplots(1, 3,
                                            figsize=(30, 12))
                    fig.subplots_adjust(wspace=0.8)

                    # 1er plot
                    plotdabest = dabest.load(data=df_lm,
                                             x="GenoA",
                                             y="new_totallength",
                                             idx=list(genos),
                                             id_col="new_cage")

                    # Tracer le graphique
                    plotdabest.mean_diff.plot(ax=axs[0],
                                              color_col='new_cage')
                    axs[0].set_title(f"""Sum of totalLength per Cage for the '{drop_event.value}' event""",
                                     loc='left')

                    # 2ème plot
                    if len(drop_injection_stat.value) == 2:
                        ax1 = two_groups_paired_baseline.mean_diff.plot(color_col='new_cage',
                                                                        ax=axs[1])
                        axs[1].legend(title='Cage',
                                      labels=result_df['new_cage'].unique(),
                                      frameon=False,
                                      loc=(1.62, 0.75))
                    elif len(drop_injection_stat.value) > 2:
                        ax1 = two_groups_paired_baseline.mean_diff.plot(color_col='new_cage',
                                                                        ax=axs[1])
                        axs[1].legend(title='Cage',
                                      labels=result_df['new_cage'].unique(),
                                      frameon=False,
                                      loc=(0.95, 0.75))

                    # 3ème plot
                    axs[2].axis('off')
                    table0 = tabulate(ssdd,
                                      headers="keys",
                                      colalign=("center", "center", "center", "center", "center"))
                    axs[2].annotate(table0,
                                    xy=(-0.45, 0.75),
                                    xycoords='axes fraction',
                                    fontsize=10, va='top',
                                    family='monospace')
                    table1 = tabulate(result.summary().tables[0],
                                      colalign=("center", "center", "center", "center", "center"))
                    axs[2].annotate(table1,
                                    xy=(-0.45, 0.55),
                                    xycoords='axes fraction',
                                    fontsize=10,
                                    va='top',
                                    family='monospace')
                    table2 = tabulate(result.summary().tables[1],
                                      headers="keys",
                                      tablefmt="plain",
                                      colalign=("center", "center", "center", "center", "center", "center", "center"))
                    axs[2].annotate(table2,
                                    xy=(-0.45, 0.35),
                                    xycoords='axes fraction',
                                    fontsize=10,
                                    va='top',
                                    family='monospace')
                    image_file_path = os.path.join(new_folder_path, f'{drop_event.value}.png')
                    plt.savefig(image_file_path, dpi=300, bbox_inches='tight')
                    plt.close(fig)

                    df_lm = df_lm.rename(columns={'new_cage': 'Cage', 'new_totallength': 'Totallength'})
        else :
            return
    else :
        return

def Start_Analyzis():
    # Launch to analyze the data
    display(tab1)
    interactive_plot = widgets.interactive_output(update_dropdown,
                                                  {'date': drop_injection_plot,
                                                   'cage': drop_cage_plot,
                                                   # 'night_phase': drop_night_plot,
                                                   'event': drop_event,
                                                   'range_slide_plot': range_slide_plot})
    display(interactive_plot)

    interactive_stats = widgets.interactive_output(update_stats,
                                                   {'date': drop_injection_stat,
                                                    'genos': genos,
                                                    'cage': drop_cage_stat,
                                                    # 'night_phase': drop_night_stat,
                                                    'event': drop_event,
                                                    'choice_type': choicetype,
                                                    'range_slide_stats': range_slide_stats})
    display(interactive_stats)


# def results_update_stats(date, genos, cage, night_phase, event, choice_type, range_slide_stats):
#     '''
#     Same figure as the 'update_stats'
#     This function is just for the widget 'statbutton' that will return plot of each values in the widget 'drop_event' in a new folder
#     named by the user. Relaunch the cell if another folder need to be created
#     '''
#     for r_drop_event in drop_event.options:
#         clear_output()
#         new_column_names_clk = []
#         ids = [i for i in df.GenoA.unique()]
#         min_value_stats, max_value_stats = range_slide_stats
#         dfs_clk = []
#         if drop_stat.value == "Linear Mixed Model":
#             # Créer un nouveau dossier pour les fichiers
#             new_folder_path = str(nom_dossier_stats.value)
#             if not os.path.exists(new_folder_path):
#                 os.makedirs(new_folder_path)
#             if choicetype.value == 'Number of events':
#                 for date_val, genos_val, cage_val, night_phase_val in itertools.product(date, genos, cage, night_phase):
#                     temp_df_loop_clk = df[(df["Injection"] == date_val)
#                                           & (df["GenoA"] == genos_val)
#                                           & (df["Cage"] == cage_val)
#                                           & (df["name"] == r_drop_event)
#                                           & (df["Night-Phase"] == night_phase_val)]
#                     # Créer un nouveau dataframe avec toutes les colonnes de l'ancien dataframe
#                     df_new_clk = temp_df_loop_clk.copy()
#                     # Ajouter une colonne avec la somme des numberOfEvents de chaque RFidA de chaque Bin
#                     df_new_clk['new_numbersOfEvents'] = df_new_clk.groupby(['Bin', 'RFidA'])[
#                         'numberOfEvents'].transform('sum')
#                     # Supprimer les lignes utilisées pour faire la somme
#                     df_new_clk = df_new_clk.drop_duplicates(subset=['Bin', 'RFidA'], keep='last')
#                     df_new_clk = df_new_clk[
#                         (df_new_clk['Bin'] >= min_value_stats) & (df_new_clk['Bin'] <= max_value_stats)]
#                     dfs_clk.append(df_new_clk)
#                 merged_df_clk = pd.concat(dfs_clk)
#                 merged_df_clk['new_cage'] = merged_df_clk['Cage'].str.extract('(\d+)').astype(int)
#                 df_lm_clk = merged_df_clk.copy()
#                 df_lm_clk = merged_df_clk.groupby(['new_cage', 'RFidA',
#                                                    'GenoA', 'Date'])['new_numbersOfEvents'].sum().reset_index()
#                 ssdd_clk = statsmodels.stats.descriptivestats.describe(df_lm_clk,
#                                                                        stats=['mean', 'std_err', 'std', 'median'])
#                 model_clk = smf.mixedlm("new_numbersOfEvents ~ GenoA",
#                                         df_lm_clk,
#                                         groups='new_cage')  # Creates the model
#                 result_clk = model_clk.fit()  # Run model
#
#                 df_lm_clk['Dates'] = df_lm_clk.groupby('RFidA')['Date'].transform(lambda x: ', '.join(x.astype(str)))
#                 # Utilisez groupby et cumcount pour attribuer des valeurs "1", "2" ou "3" en fonction de l'ordre des dates pour chaque RFidA
#                 df_lm_clk['New_Column'] = df_lm_clk.groupby('RFidA').cumcount() + 1
#                 df_lm_clk['New_Column'] = df_lm_clk['New_Column'].astype(str)
#                 df_lm_clk = df_lm_clk.sort_values(by=['RFidA', 'new_cage', 'Date']).reset_index(drop=True)
#
#                 new_df_clk = df_lm_clk.loc[:, ['new_cage', 'RFidA']]
#
#                 # Récupérez le nombre de sélections dans le widget
#                 num_selections_clk = len(drop_injection_stat.value)
#
#                 # Récupérez les valeurs de sélections dans le widget
#                 name_selections_clk = list(drop_injection_stat.value)
#
#                 num_select_clk = len(name_selections_clk)
#
#                 # Ajoutez une colonne 'Injection' au DataFrame
#                 df_lm_clk['Injection'] = [name_selections_clk[i % num_select_clk] for i in range(len(df_lm_clk))]
#
#                 # Créez un dictionnaire pour stocker les données pivotées
#                 pivot_data_clk = {'new_cage': [], 'RFidA': []}
#                 for injection in name_selections_clk:
#                     pivot_data_clk[injection] = []
#
#                 # Parcourez les lignes du premier DataFrame
#                 for _, row in df_lm_clk.iterrows():
#                     liste_newcage_clk = row['new_cage']
#                     liste_RFidA_clk = row['RFidA']
#                     liste_injection_clk = row['Injection']
#                     liste_nbevents_clk = row['new_numbersOfEvents']
#
#                     # Vérifiez si la valeur de 'Injection' est dans la liste sélectionnée
#                     if liste_injection_clk in name_selections_clk:
#                         pivot_data_clk['new_cage'].append(liste_newcage_clk)
#                         pivot_data_clk['RFidA'].append(liste_RFidA_clk)
#                         for selected_injection in name_selections_clk:
#                             if selected_injection == liste_injection_clk:
#                                 pivot_data_clk[selected_injection].append(liste_nbevents_clk)
#                             else:
#                                 pivot_data_clk[selected_injection].append(None)
#
#                 # Créez un DataFrame à partir du dictionnaire pivoté
#                 pivot_df_clk = pd.DataFrame(pivot_data_clk)
#
#                 # Fusionnez df2 avec le DataFrame pivoté
#                 result_df_clk = pd.merge(new_df_clk, pivot_df_clk, on=['new_cage', 'RFidA'], how='left')
#
#                 # Groupez le résultat par 'new_cage' et 'RFidA' et agrégez les valeurs
#                 result_df_clk = result_df_clk.groupby(['new_cage', 'RFidA'])[name_selections_clk].first().reset_index()
#
#                 # Remplacez les valeurs NaN par 'None'
#                 result_df_clk = result_df_clk.fillna('0')
#                 result_df_clk.reset_index(drop=True, inplace=True)
#                 result_df_clk.insert(0, 'Index', range(len(result_df_clk)))
#
#                 # Réinitialisez l'index du deuxième DataFrame
#                 new_df_clk = new_df_clk.reset_index(drop=True)
#
#                 # Convertir toutes les colonnes sauf 'Index', 'new_cage', et 'RFidA' en int64
#                 result_df_clk[result_df_clk.columns.difference(['Index', 'new_cage', 'RFidA'])] = result_df_clk[
#                     result_df_clk.columns.difference(['Index', 'new_cage', 'RFidA'])].astype('int64')
#                 result_df_clk
#                 pivot_df2_clk = df_lm_clk.pivot(index=['RFidA', 'new_cage'],
#                                                 columns='Date',
#                                                 values='new_numbersOfEvents').reset_index()
#                 pivot_df2_clk.sort_values('new_cage',
#                                           inplace=True)
#
#                 two_groups_paired_baseline_clk = dabest.load(data=result_df_clk,
#                                                              idx=list(drop_injection_stat.value),
#                                                              id_col="Index",
#                                                              paired='baseline')
#
#                 fig, axs = plt.subplots(1, 3,
#                                         figsize=(30, 12))
#                 fig.subplots_adjust(wspace=0.8)
#
#                 # 1er plot
#                 plotdabest_clk = dabest.load(data=df_lm_clk,
#                                              x="GenoA",
#                                              y="new_numbersOfEvents",
#                                              idx=list(genos),
#                                              id_col="new_cage")
#
#                 # Tracer le graphique
#                 plotdabest_clk.mean_diff.plot(ax=axs[0],
#                                               color_col='new_cage')
#                 axs[0].set_title(f"""Sum of numberOfEvents per Cage for the '{drop_event.value}' event""",
#                                  loc='left')
#
#                 # 2ème plot
#                 if len(drop_injection_stat.value) == 2:
#                     ax1 = two_groups_paired_baseline_clk.mean_diff.plot(color_col='new_cage',
#                                                                         ax=axs[1])
#                     axs[1].legend(title='Cage',
#                                   labels=result_df_clk['new_cage'].unique(),
#                                   frameon=False,
#                                   loc=(1.62, 0.75))
#                 elif len(drop_injection_stat.value) > 2:
#                     ax1 = two_groups_paired_baseline_clk.mean_diff.plot(color_col='new_cage',
#                                                                         ax=axs[1])
#                     axs[1].legend(title='Cage',
#                                   labels=result_df_clk['new_cage'].unique(),
#                                   frameon=False,
#                                   loc=(0.95, 0.75))
#
#                 # 3ème plot
#                 axs[2].axis('off')
#                 table0 = tabulate(ssdd_clk,
#                                   headers="keys",
#                                   colalign=("center", "center", "center", "center", "center"))
#                 axs[2].annotate(table0,
#                                 xy=(-0.45, 0.75),
#                                 xycoords='axes fraction',
#                                 fontsize=10, va='top',
#                                 family='monospace')
#                 table1 = tabulate(result_clk.summary().tables[0],
#                                   colalign=("center", "center", "center", "center", "center"))
#                 axs[2].annotate(table1,
#                                 xy=(-0.45, 0.55),
#                                 xycoords='axes fraction',
#                                 fontsize=10,
#                                 va='top',
#                                 family='monospace')
#                 table2 = tabulate(result_clk.summary().tables[1],
#                                   headers="keys",
#                                   tablefmt="plain",
#                                   colalign=("center", "center", "center", "center", "center", "center", "center"))
#                 axs[2].annotate(table2,
#                                 xy=(-0.45, 0.35),
#                                 xycoords='axes fraction',
#                                 fontsize=10,
#                                 va='top',
#                                 family='monospace')
#                 image_file_path = os.path.join(new_folder_path, f'{r_drop_event}.png')
#                 plt.savefig(image_file_path,
#                             dpi=300,
#                             bbox_inches='tight')
#                 plt.close(fig)
#             elif choicetype.value == 'Event duration':
#                 for date_val, cage_val, night_phase_val in itertools.product(date, cage, night_phase):
#                     temp_df_loop_clk = df[(df["Injection"] == date_val)
#                                           & (df["Cage"] == cage_val)
#                                           & (df["name"] == r_drop_event)
#                                           & (df["Night-Phase"] == night_phase_val)]
#                     # Créer un nouveau dataframe avec toutes les colonnes de l'ancien dataframe
#                     df_new_clk = temp_df_loop_clk.copy()
#                     # Ajouter une colonne avec la somme des numberOfEvents de chaque RFidA de chaque Bin
#                     df_new_clk['new_totallength'] = df_new_clk.groupby(['Bin',
#                                                                         'RFidA'])['totalLength'].transform('sum')
#                     # Supprimer les lignes utilisées pour faire la somme
#                     df_new_clk = df_new_clk.drop_duplicates(subset=['Bin', 'RFidA'],
#                                                             keep='last')
#                     df_new_clk = df_new_clk[
#                         (df_new_clk['Bin'] >= min_value_stats) & (df_new_clk['Bin'] <= max_value_stats)]
#                     dfs_clk.append(df_new_clk)
#                 merged_df_clk = pd.concat(dfs_clk)
#                 merged_df_clk['new_cage'] = merged_df_clk['Cage'].str.extract('(\d+)').astype(int)
#                 df_lm_clk = merged_df_clk.copy()
#                 df_lm_clk = merged_df_clk.groupby(['new_cage', 'RFidA',
#                                                    'GenoA', 'Date'])['new_totallength'].sum().reset_index()
#                 ssdd_clk = statsmodels.stats.descriptivestats.describe(df_lm_clk,
#                                                                        stats=['mean', 'std_err', 'std', 'median'])
#                 model_clk = smf.mixedlm("new_totallength ~ GenoA",
#                                         df_lm_clk,
#                                         groups='new_cage')  # Creates the model
#                 result_clk = model_clk.fit()  # Run model
#
#                 df_lm_clk['Dates'] = df_lm_clk.groupby('RFidA')['Date'].transform(lambda x: ', '.join(x.astype(str)))
#                 # Utilisez groupby et cumcount pour attribuer des valeurs "1", "2" ou "3" en fonction de l'ordre des dates pour chaque RFidA
#                 df_lm_clk['New_Column'] = df_lm_clk.groupby('RFidA').cumcount() + 1
#                 df_lm_clk['New_Column'] = df_lm_clk['New_Column'].astype(str)
#                 df_lm_clk = df_lm_clk.sort_values(by=['RFidA', 'new_cage', 'Date']).reset_index(drop=True)
#
#                 new_df_clk = df_lm_clk.loc[:, ['new_cage', 'RFidA']]
#
#                 # Récupérez le nombre de sélections dans le widget
#                 num_selections = len(drop_injection_stat.value)
#
#                 # Récupérez les valeurs de sélections dans le widget
#                 name_selections_clk = list(drop_injection_stat.value)
#
#                 num_select_clk = len(name_selections_clk)
#
#                 # Ajoutez une colonne 'Injection' au DataFrame
#                 df_lm_clk['Injection'] = [name_selections_clk[i % num_select_clk] for i in range(len(df_lm_clk))]
#
#                 # Créez un dictionnaire pour stocker les données pivotées
#                 pivot_data_clk = {'new_cage': [], 'RFidA': []}
#                 for injection in name_selections_clk:
#                     pivot_data_clk[injection] = []
#
#                 # Parcourez les lignes du premier DataFrame
#                 for _, row in df_lm_clk.iterrows():
#                     liste_newcage_clk = row['new_cage']
#                     liste_RFidA_clk = row['RFidA']
#                     liste_injection_clk = row['Injection']
#                     liste_nbevents_clk = row['new_totallength']
#
#                     # Vérifiez si la valeur de 'Injection' est dans la liste sélectionnée
#                     if liste_injection_clk in name_selections_clk:
#                         pivot_data_clk['new_cage'].append(liste_newcage_clk)
#                         pivot_data_clk['RFidA'].append(liste_RFidA_clk)
#                         for selected_injection in name_selections_clk:
#                             if selected_injection == liste_injection_clk:
#                                 pivot_data_clk[selected_injection].append(liste_nbevents_clk)
#                             else:
#                                 pivot_data_clk[selected_injection].append(None)
#
#                 # Créez un DataFrame à partir du dictionnaire pivoté
#                 pivot_df_clk = pd.DataFrame(pivot_data_clk)
#
#                 # Fusionnez df2 avec le DataFrame pivoté
#                 result_df_clk = pd.merge(new_df_clk, pivot_df_clk, on=['new_cage', 'RFidA'], how='left')
#
#                 # Groupez le résultat par 'new_cage' et 'RFidA' et agrégez les valeurs
#                 result_df_clk = result_df_clk.groupby(['new_cage', 'RFidA'])[name_selections_clk].first().reset_index()
#
#                 # Remplacez les valeurs NaN par 'None'
#                 result_df_clk = result_df_clk.fillna('0')
#                 result_df_clk.reset_index(drop=True, inplace=True)
#                 result_df_clk.insert(0, 'Index', range(len(result_df_clk)))
#
#                 # Réinitialisez l'index du deuxième DataFrame
#                 new_df_clk = new_df_clk.reset_index(drop=True)
#
#                 # Convertir toutes les colonnes sauf 'Index', 'new_cage', et 'RFidA' en int64
#                 result_df_clk[result_df_clk.columns.difference(['Index', 'new_cage', 'RFidA'])] = result_df_clk[
#                     result_df_clk.columns.difference(['Index', 'new_cage', 'RFidA'])].astype('int64')
#                 result_df_clk
#                 pivot_df2_clk = df_lm_clk.pivot(index=['RFidA', 'new_cage'],
#                                                 columns='Date',
#                                                 values='new_totallength').reset_index()
#                 pivot_df2_clk.sort_values('new_cage',
#                                           inplace=True)
#
#                 two_groups_paired_baseline = dabest.load(data=result_df_clk,
#                                                          idx=list(drop_injection_stat.value),
#                                                          id_col="Index",
#                                                          paired='baseline')
#
#                 fig, axs = plt.subplots(1, 3,
#                                         figsize=(30, 12))
#                 fig.subplots_adjust(wspace=0.8)
#
#                 # 1er plot
#                 plotdabest = dabest.load(data=df_lm_clk,
#                                          x="GenoA",
#                                          y="new_totallength",
#                                          idx=list(genos),
#                                          id_col="new_cage")
#
#                 # Tracer le graphique
#                 plotdabest.mean_diff.plot(ax=axs[0],
#                                           color_col='new_cage')
#                 axs[0].set_title(f"""Sum of totalLength per Cage for the '{drop_event.value}' event""",
#                                  loc='left')
#
#                 # 2ème plot
#                 if len(drop_injection_stat.value) == 2:
#                     ax1 = two_groups_paired_baseline.mean_diff.plot(color_col='new_cage',
#                                                                     ax=axs[1])
#                     axs[1].legend(title='Cage',
#                                   labels=result_df_clk['new_cage'].unique(),
#                                   frameon=False,
#                                   loc=(1.62, 0.75))
#                 elif len(drop_injection_stat.value) > 2:
#                     ax1 = two_groups_paired_baseline.mean_diff.plot(color_col='new_cage',
#                                                                     ax=axs[1])
#                     axs[1].legend(title='Cage',
#                                   labels=result_df_clk['new_cage'].unique(),
#                                   frameon=False,
#                                   loc=(0.95, 0.75))
#
#                 # 3ème plot
#                 axs[2].axis('off')
#                 table0 = tabulate(ssdd_clk,
#                                   headers="keys",
#                                   colalign=("center", "center", "center", "center", "center"))
#                 axs[2].annotate(table0,
#                                 xy=(-0.45, 0.75),
#                                 xycoords='axes fraction',
#                                 fontsize=10, va='top',
#                                 family='monospace')
#                 table1 = tabulate(result_clk.summary().tables[0],
#                                   colalign=("center", "center", "center", "center", "center"))
#                 axs[2].annotate(table1,
#                                 xy=(-0.45, 0.55),
#                                 xycoords='axes fraction',
#                                 fontsize=10,
#                                 va='top',
#                                 family='monospace')
#                 table2 = tabulate(result_clk.summary().tables[1],
#                                   headers="keys",
#                                   tablefmt="plain",
#                                   colalign=("center", "center", "center", "center", "center", "center", "center"))
#                 axs[2].annotate(table2,
#                                 xy=(-0.45, 0.35),
#                                 xycoords='axes fraction',
#                                 fontsize=10,
#                                 va='top',
#                                 family='monospace')
#                 image_file_path = os.path.join(new_folder_path, f'{r_drop_event}.png')
#                 plt.savefig(image_file_path,
#                             dpi=300,
#                             bbox_inches='tight')
#                 plt.close(fig)
#             else:
#                 return
#         else:
#             return

# interactive_plot = widgets.interactive_output(update_dropdown,
#                                               {'date': drop_injection_plot,
#                                                'cage': drop_cage_plot,
#                                                'night_phase': drop_night_plot,
#                                                'event': drop_event})
# display(interactive_plot)
#
# interactive_stats = widgets.interactive_output(update_stats,
#                                                {'date': drop_injection_stat,
#                                                 'cage': drop_cage_stat,
#                                                 'night_phase': drop_night_stat,
#                                                 'event': drop_event,
#                                                 'choice_type':choicetype})
# display(interactive_stats)



'''
Created on 23 Feb. 2022

@author: Elodie
'''


exclusiveEventList = ['Oral-oral Contact exclusive', 'Side by side Contact exclusive', 'Oral-genital Contact exclusive',
                       'Passive oral-genital Contact exclusive', 'Side by side Contact, opposite way exclusive',
                       'Oral-oral and Side by side Contact exclusive',
                       'Oral-genital and Side by side Contact, opposite way exclusive',
                       'Oral-genital passive and Side by side Contact, opposite way exclusive', 'Other contact exclusive', 'Move isolated exclusive', 'Stop isolated exclusive', 'Undetected exclusive']

contactTypeList = ["Oral-oral Contact", "Oral-genital Contact", "Passive oral-genital Contact", "Side by side Contact", "Side by side Contact, opposite way", "Other contact"]

exclusiveEventsLabels = {'Oral-oral Contact exclusive': 'N-N', 'Side by side Contact exclusive': 'S-S', 'Oral-genital Contact exclusive': 'N-G',
                       'Passive oral-genital Contact exclusive': 'N-G pass', 'Side by side Contact, opposite way exclusive': 'S-S opp',
                       'Oral-oral and Side by side Contact exclusive': 'N-N S-S',
                       'Oral-genital and Side by side Contact, opposite way exclusive': 'N-G S-S opp',
                       'Oral-genital passive and Side by side Contact, opposite way exclusive': 'N-G pass S-S opp', 'Other contact exclusive': 'other',
                        'Move isolated exclusive': 'move', 'Stop isolated exclusive': 'idle', 'Undetected exclusive': 'undetected'}

exclusiveEventListLabels = [exclusiveEventsLabels[x] for x in exclusiveEventList]

sexList = ['male', 'female']
mutantGeno = 'Del/+'
genoList = ['WT', mutantGeno]

genoListGeneral = ['{}-{}'.format(genoList[0], genoList[0]), '{}-{}'.format(genoList[0], genoList[1]),
                   '{}-{}'.format(genoList[1], genoList[1])]
sexListGeneral = ['{}-{}'.format(sexList[0], sexList[0]), '{}-{}'.format(sexList[0], sexList[1]),
                  '{}-{}'.format(sexList[1], sexList[1])]



'''
Created on 6 sept. 2017

@author: Fab
'''
import sqlite3
from time import *
from lmtanalysis.Chronometer import Chronometer
from lmtanalysis.Animal import *
from lmtanalysis.Detection import *
from lmtanalysis.Measure import *
import matplotlib.pyplot as plt
import numpy as np
from lmtanalysis.Event import *
from lmtanalysis.Measure import *
from lmtanalysis.EventTimeLineCache import EventTimeLineCached

def flush( connection ):
    ''' flush event in database '''
    deleteEventTimeLineInBase(connection, "Group4" )


def reBuildEvent( connection, file, tmin=None, tmax=None , pool = None ):
    '''
    four animals are in contact. (equivalent to group2 and group3)
    ''' 
    
    pool = AnimalPool( )
    pool.loadAnimals( connection )
    #pool.loadDetection( start = tmin, end = tmax )
    
    
    contact = {}
    group2 = {}
    for animal in range( 1 , 5 ):    
        contact[animal] = EventTimeLineCached( connection, file, "Contact", animal, minFrame=tmin, maxFrame=tmax ) #fait une matrice de tous les contacts à deux possibles
        group2[animal] = EventTimeLineCached(connection, file, "Group2", animal, minFrame=tmin, maxFrame=tmax )
        
    for animal in range( 1 , 5 ):
        
        for idAnimalB in range( 1 , 5 ):
            if( animal == idAnimalB ):
                continue
            
            for idAnimalC in range( 1 , 5 ):
                if( animal == idAnimalC ):
                    continue
                if( idAnimalB == idAnimalC ):
                    continue
                
                for idAnimalD in range( 1 , 5 ):
                    if( animal == idAnimalD ):
                        continue
                    if( idAnimalB == idAnimalD ):
                        continue
                    if( idAnimalC == idAnimalD ):
                        continue
                
                    eventName = "Group4"        
                    # print ( eventName )
                    
                    groupTimeLine = EventTimeLine( None, eventName , animal , idAnimalB , idAnimalC , idAnimalD , loadEvent=False )
                    
                    result={}
                    
                    dicA = contact[ animal ].getDictionnary()
                    dicB = contact[ idAnimalB ].getDictionnary()
                    dicC = contact[ idAnimalC ].getDictionnary()
                    dicD = contact[ idAnimalD ].getDictionnary()
                    
                    dicGroup2A = group2[ animal ].getDictionnary()
                    dicGroup2B = group2[ idAnimalB ].getDictionnary()
                    dicGroup2C = group2[ idAnimalC ].getDictionnary()
                    dicGroup2D = group2[ idAnimalD ].getDictionnary()
                    
                    for t in dicA.keys():
                        if ( t in dicB and t in dicC and t in dicD ):
                            if ( t in dicGroup2A or t in dicGroup2B or t in dicGroup2C or t in dicGroup2D):
                                continue
                            else:
                                result[t]=True
                    
                    
    groupTimeLine.reBuildWithDictionnary( result )
    
    groupTimeLine.endRebuildEventTimeLine(connection)
          
        
    # log process
    from lmtanalysis.TaskLogger import TaskLogger
    t = TaskLogger( connection )
    t.addLog( "Build Event Group4" , tmin=tmin, tmax=tmax )
          
    
    # print( "Rebuild event finished." )
        
    


'''
Created on 6 sept. 2017

@author: Fab
'''
import sqlite3
from time import *
from lmtanalysis.Chronometer import Chronometer
from lmtanalysis.Animal import *
from lmtanalysis.Detection import *
from lmtanalysis.Measure import *
import matplotlib.pyplot as plt
import numpy as np
from lmtanalysis.Event import *
from lmtanalysis.EventTimeLineCache import EventTimeLineCached

class Train2():
    
    def __init__(self, idA, idB ):
        self.idA = idA
        self.idB = idB

def flush( connection ):
    ''' flush event in database '''
    deleteEventTimeLineInBase(connection, "Train4" )

def reBuildEvent( connection, file, tmin=None, tmax=None , pool = None ): 

    ''' use pool cache if available '''
    if ( pool == None ):
        pool = AnimalPool( )
        pool.loadAnimals( connection )
        if ( len ( pool.getAnimalList( ) ) < 4 ):
            # print("Train 4 cannot be computed on an experiment with less than 4 animals")
            return
        pool.loadDetection( start = tmin, end = tmax )    
    
    '''
    three animals are following each others with nose-to-anogenital contacts
    animals are moving
    
    this is a combination of Train2 events (train2 events must be calculated before this event)
    '''

    #deleteEventTimeLineInBase(connection, "Train4" )

    ''' build a list of train 2 for each time point '''
    
    time = {}
    train4 = {}
    
    for animal in range( 1, pool.getNbAnimals()+1 ):
        for idAnimalB in range( 1 , pool.getNbAnimals()+1 ):
            if ( animal == idAnimalB ):
                continue
            train2TimeLine = EventTimeLineCached( connection, file, "Train2", animal, idAnimalB, minFrame=tmin, maxFrame=tmax )
            for t in train2TimeLine.getDictionnary():
                train = Train2( animal, idAnimalB )
                
                if ( not t in time ):
                    time[t] =[]
                    
                #print ( t , ":" , train.idA , " -> ", train.idB , "*" )
                time[t].append( train )
    

    for t in time:
        trainList = time[t]
        
        for train1st in trainList:
            
            for train2nd in trainList:
                
                if ( train1st == train2nd ):
                    continue

                for train3rd in trainList:
                    
                    if ( train3rd == train1st or train3rd == train2nd):
                        continue
                    
                    isValid = ""
                    ''' test chain link between train2 events '''
                   
                    if train1st.idB == train2nd.idA and train2nd.idB == train3rd.idA :
                        
                        id1 = train1st.idA
                        id2 = train1st.idB
                        id3 = train2nd.idB
                        id4 = train3rd.idB
    
                        if not (id1, id2, id3,id4) in train4:    
                            train4[id1,id2,id3,id4] = {} 
                        
                        train4[id1,id2,id3,id4][t]=True
                        
                        isValid = ": validated train 4"
                    #print ( t , ":" , train1st.idA , " -> ", train1st.idB, "--->", train2nd.idA , " -> ", train2nd.idB , " -> ", train3rd.idA , " -> ", train3rd.idB , isValid )


    ''' save data '''
            
    for animal in range( 1 , pool.getNbAnimals()+1 ):
    
        for idAnimalB in range( 1 , pool.getNbAnimals()+1 ):
            
            for idAnimalC in range( 1 , pool.getNbAnimals()+1 ):

                for idAnimalD in range( 1 , pool.getNbAnimals()+1 ):

                    if (animal, idAnimalB, idAnimalC, idAnimalD) in train4:
                        
                        trainTimeLine = EventTimeLine( None, "Train4" , animal , idAnimalB, idAnimalC, idAnimalD, loadEvent=False )
                        
                        trainTimeLine.reBuildWithDictionnary( train4[animal,idAnimalB,idAnimalC,idAnimalD] )
                        #trainTimeLine.removeEventsBelowLength( 5 )            
                        trainTimeLine.endRebuildEventTimeLine(connection)
                    
        
    # log process
    from lmtanalysis.TaskLogger import TaskLogger
    t = TaskLogger( connection )
    t.addLog( "Build Event Train4" , tmin=tmin, tmax=tmax )

    # print( "Rebuild event finished." )
    
    return
   
    
    


'''
Created on 6 sept. 2017

@author: Fab
'''
import sqlite3
from time import *
from lmtanalysis.Chronometer import Chronometer
from lmtanalysis.Animal import *
from lmtanalysis.Detection import *
from lmtanalysis.Measure import *
import matplotlib.pyplot as plt
import numpy as np
from lmtanalysis.Event import *
from lmtanalysis.Measure import *
from lmtanalysis.EventTimeLineCache import EventTimeLineCached
from lmtanalysis.BehaviouralSequencesUtil import exclusiveEventList

def flush( connection ):
    ''' flush event in database '''
    eventUndetected = 'Undetected exclusive'
    deleteEventTimeLineInBase(connection, eventUndetected)

def reBuildEvent( connection, file, tmin=None, tmax=None , pool = None ):
    eventUndetected = 'Undetected exclusive'
    if ( pool == None ):
        pool = AnimalPool( )
        pool.loadAnimals( connection )

    ''' load the existing detection timeline for animal '''
    detectionTimeLine = {}
    dicoDetection = {}
    undetectedTimeLine = {}
    for animal in range(1, pool.getNbAnimals() + 1):
        detectionTimeLine[animal] = EventTimeLineCached( connection, file, 'Detection', animal, minFrame=tmin, maxFrame=tmax )
        dicoDetection[animal] = detectionTimeLine[animal].getDictionary(minFrame=tmin, maxFrame=tmax)
        undetectedTimeLine[animal] = EventTimeLine(None, eventUndetected, animal, loadEvent=False)

    '''select the frames where the animals are not detected'''
    undetectedDico = {}
    for animal in range(1, pool.getNbAnimals() + 1):
        undetectedDico[animal] = {}
        for t in range(tmin, tmax+1):
            if t not in dicoDetection[animal]:
                undetectedDico[animal][t] = True


    #####################################################
    #reduild all events based on dictionary
    for animal in range(1, pool.getNbAnimals() + 1):
        undetectedTimeLine[animal].reBuildWithDictionnary(undetectedDico[animal])
        undetectedTimeLine[animal].endRebuildEventTimeLine(connection)

    # log process
    from lmtanalysis.TaskLogger import TaskLogger
    t = TaskLogger( connection )
    t.addLog( "Build Event Exclusive undetected events" , tmin=tmin, tmax=tmax )
                       
    # print( "Rebuild event finished." )
        
    


'''
Created on 6 sept. 2017

@author: Fab
'''

import math

class Point:
    '''
    classdocs
    '''


    __slots__ = ('x', 'y' )


    def __init__(self, x,y):
        self.x = x;
        self.y = y;
        
    def distanceTo(self , p ):
        return math.hypot( self.x-p.x , self.y-p.y )
        
        


'''
Created on 13 sept. 2017

@author: Fab
'''
from lmtanalysis.Point import Point

''' scale to convert distances from pixels to cm '''
scaleFactor = 10/57

''' '''
SPEED_THRESHOLD_LOW = 5
SPEED_THRESHOLD_HIGH = 10

''' slope of the body between the nose and the tail basis '''
BODY_SLOPE_THRESHOLD = 40

''' threshold for the distance contact using mass center between two detection '''
DISTANCE_CONTACT_MASS_CENTER = 8/scaleFactor

''' threshold for the maximum distance allowed between two points '''
MAX_DISTANCE_THRESHOLD = 71/scaleFactor

''' threshold min time after a first rearing to detect the second rearing (frames)'''
SEQUENTIAL_REARING_MIN_TIME_THRESHOLD = 10

''' threshold max time after a first rearing to detect the second rearing (frames)'''
SEQUENTIAL_REARING_MAX_TIME_THRESHOLD = 30

''' threshold for the area around a reared animal for sequential rearing '''
SEQUENTIAL_REARING_POSITION_THRESHOLD = 50

''' threshold for the presence within the water zone '''
MAX_DISTANCE_TO_POINT = 5/scaleFactor

''' threshold to classify the detection as a rearing; height of the front point'''
FRONT_REARING_THRESHOLD = 50

''' threshold to compute head-genital events '''
MAX_DISTANCE_HEAD_HEAD_GENITAL_THRESHOLD = 15

oneFrame = 1
oneSecond = 30
oneMinute = 30*60
oneHour = 30*60*60
oneDay = 30*60*60*24
oneWeek = 30*60*60*24*7

'''time window at the end of an event to test overlap with another event'''
TIME_WINDOW_BEFORE_EVENT = 15*oneFrame

''' Minimum duration for the animal to stop at the water point to be classified as drinking '''
MIN_WATER_STOP_DURATION = 2*oneSecond

''' Cage center in 50x50cm area'''
cageCenterCoordinates50x50Area = Point( 256, 208 )

''' Size of arena in cm '''
ARENA_SIZE = 50

''' Margin to define center region in cm (chosen to have same area as non-center)'''
CENTER_MARGIN = 7.32

''' Corner Coordinates in 50x50cm area '''
cornerCoordinates50x50Area = [
                        (114,63),
                        (398,63),
                        (398,353),
                        (114,353)
                        ]

def second( second ):
    return second * oneSecond

def day( day ):
    return day* oneDay

def hour( hour ):
    return hour * oneHour